import cmor
import numpy

ntimes = 1
nlat = 45
nlon = 90
nlev = 5


def prep(mode):
    error_flag = cmor.setup(
        inpath='Tables',
        netcdf_file_action=mode)

    error_flag = cmor.dataset_json("Test/CMOR_input_example.json")


def prep_var(var, units):
    # creates 1 degree grid
    dlat = 180 / nlat
    dlon = 360. / nlon
    alats = numpy.arange(-90 + dlat / 2., 90, dlat)
    bnds_lat = numpy.arange(-90, 90 + dlat, dlat)
    alons = numpy.arange(0 + dlon / 2., 360., dlon) - 180.
    bnds_lon = numpy.arange(0, 360. + dlon, dlon) - 180.
    cmor.load_table("Tables/CMIP6_6hrLev.json")
    # cmor.load_table("Test/IPCC_table_A1")
    ilat = cmor.axis(
        table_entry='latitude',
        units='degrees_north',
        length=nlat,
        coord_vals=alats,
        cell_bounds=bnds_lat)

    ilon = cmor.axis(
        table_entry='longitude',
        length=nlon,
        units='degrees_east',
        coord_vals=alons,
        cell_bounds=bnds_lon)

    zlevs = numpy.zeros(5, dtype='d')
    zlevs[0] = 0.1999999999999999999
    zlevs[1] = 0.3
    zlevs[2] = 0.55
    zlevs[3] = 0.7
    zlevs[4] = 0.99999999

    zlev_bnds = numpy.zeros(6, dtype='d')
    zlev_bnds[0] = 0.
    zlev_bnds[1] = 0.2
    zlev_bnds[2] = 0.42
    zlev_bnds[3] = 0.62
    zlev_bnds[4] = 0.8
    zlev_bnds[5] = 1.

    itim = cmor.axis(
        table_entry='time1',
        units='days since 2010-1-1')

    ilev = cmor.axis(
        table_entry="alternate_hybrid_sigma",
        units='1',
        coord_vals=zlevs,
        cell_bounds=zlev_bnds)

    p0 = numpy.array([1.e5, ])
    a_coeff = numpy.array([0.1, 0.2, 0.3, 0.22, 0.1])
    b_coeff = numpy.array([0.0, 0.1, 0.2, 0.5, 0.8])
    a_coeff_bnds = numpy.array([0., .15, .25, .25, .16, 0.])
    b_coeff_bnds = numpy.array([0., .05, .15, .35, .65, 1.])

    ierr = cmor.zfactor(zaxis_id=ilev,
                        zfactor_name='ap',
                        units='Pa',
                        axis_ids=[ilev, ],
                        zfactor_values=a_coeff,
                        zfactor_bounds=a_coeff_bnds)

    ierr = cmor.zfactor(zaxis_id=ilev,
                        zfactor_name='b',
                        axis_ids=[ilev, ],
                        zfactor_values=b_coeff,
                        zfactor_bounds=b_coeff_bnds)

    # ierr = cmor.zfactor(zaxis_id=ilev,
    # zfactor_name='p0',
    # units='Pa',
    # zfactor_values=p0)

    ips = cmor.zfactor(zaxis_id=ilev,
                       zfactor_name='ps1',
                       axis_ids=[itim, ilat, ilon],
                       units='Pa')

    ivar1 = cmor.variable(
        var,
        axis_ids=[
            itim,
            ilev,
            ilat,
            ilon],
        units=units,
        missing_value=0.)
    return ivar1, ips


for d in range(2):
    mode = cmor.CMOR_APPEND
    if d == 0:
        mode = cmor.CMOR_REPLACE
    prep(mode)
    ivar1, ips1 = prep_var("ta", "K")
    ivar2, ips2 = prep_var("hus", "%")
    for i in range(4):
        tval = [i / 4. + d]
        tbnd = [i / 4. + d - 0.125, i / 4. + d + 0.125]
        print('tvar', tval)
        print('tbnd', tbnd)
        print('writing time:', i, i / 4.)
        data = numpy.random.random((ntimes, nlev, nlat, nlon)) * 30. + 273
        data = data.astype("f")
        cmor.write(ivar1, data, time_vals=tval, time_bnds=tbnd)
        print('wrote var 1 time:', i)
        data = numpy.random.random((ntimes, nlev, nlat, nlon))
        data = data.astype("f")
        cmor.write(ivar2, data, time_vals=tval, time_bnds=tbnd)
        print('wrote var 2 time:', i)
        data = numpy.random.random((ntimes, nlat, nlon)) * 8. + 96300.
        data = data.astype("f")
        cmor.write(
            ips1,
            data,
            store_with=ivar1,
            ntimes_passed=1,
            time_vals=tval,
            time_bnds=tbnd)
        print('wrote ps in var 1 time:', i)
        cmor.write(
            ips2,
            data,
            store_with=ivar2,
            ntimes_passed=1,
            time_vals=tval,
            time_bnds=tbnd)
        print('wrote ps in var 2 time:', i)
    file1 = cmor.close(ivar1, True)
    file2 = cmor.close(ivar2, True)
    print('File1:', file1)
    print('File2:', file2)
    cmor.close()



import glob
import os
import unittest


def create_test_suite():
    test_file_strings = glob.glob('Test/test_python_CMIP6_CV*.py')
#    test_file_strings = glob.glob('Test/test_python_CMIP6_CV*further*.py')
    test_file_strings.extend(['Test/test_python_has_cur_dataset_attribute.py',
                              'Test/test_python_has_variable_attribute.py'])
    module_strings = [os.path.splitext(test_file_string)[0].replace('/', '.')
                      for test_file_string in test_file_strings]
    suites = [unittest.defaultTestLoader.loadTestsFromName(module_string)
              for module_string in module_strings]
    test_suite = unittest.TestSuite(suites)
    return test_suite



#!/usr/bin/env python
import cmor
import numpy


def define_axes(axes):
    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    print('cmor.axis calls complete')
    return axis_ids


def define_write_var(axis_ids, entry, unit, values):
    varid = cmor.variable(entry,
                          unit,
                          axis_ids,
                          missing_value=-99
                          )

    print('cmor.variable call complete')

    cmor.write(varid, values, time_vals=[15.0], time_bnds=[0., 30.0])

    print('cmor.write call complete')


def cmor_ini():
    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE)
    cmor.dataset_json("Test/CMOR_input_example.json")


def define_write_clisccp():
    cmor.load_table('CMIP6_CFmon.json')
    axes = [{'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            {'table_entry': 'plev7c',
             'coord_vals': [90000., 74000., 62000., 50000., 37500., 24500., 9000.],
             'cell_bounds': [[100000., 80000.], [80000., 68000.], [68000., 56000.], [56000., 44000.], [44000., 31000.], [31000., 18000.], [18000., 0.]],
             'units': 'Pa',
             },
            {'table_entry': 'tau',
             'coord_vals': [0.15, 0.8, 2.45, 6.5, 16.2, 41.5, 100.],
             'cell_bounds':[[0.0, 0.3], [0.3, 1.3], [1.3, 3.6], [3.6, 9.4], [9.4, 23.0], [23.0, 60.0], [60.0, 100000]],
             'units': '1'}
            ]

    axis_ids = define_axes(axes)

    values = numpy.array([0.0004, ] * 49, numpy.float32)
    values = numpy.reshape(values, (1, 1, 1, 7, 7))

    define_write_var(axis_ids, 'clisccp', '1', values)


def define_write_landcoverfrac():
    cmor.load_table('Tables/CMIP6_Lmon.json')
    axes = [{'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            {'table_entry': 'vegtype',
             'coord_vals': ['landcover'],
             'units': '1',
             },
            ]

    axis_ids = define_axes(axes)

    values = numpy.array([2.], numpy.float32)
    values = numpy.reshape(values, (1, 1, 1, 1))

    define_write_var(axis_ids, 'landCoverFrac', '1', values)


def main():

    cmor_ini()
    define_write_clisccp()
    define_write_landcoverfrac()
    cmor.close()


if __name__ == '__main__':

    main()



from test_python_common import *  # common subroutines

import cmor._cmor
import os

pth = os.path.split(os.path.realpath(os.curdir))
if pth[-1] == 'Test':
    ipth = opth = '.'
else:
    ipth = opth = 'Tables'


myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')


cmor.setup(
    inpath=ipth,
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE,
    exit_control=cmor.CMOR_EXIT_ON_MAJOR)
cmor.dataset_json("Test/CMOR_input_example.json")

tables = []
a = cmor.load_table("CMIP6_grids.json")
tables.append(a)
tables.append(cmor.load_table("CMIP6_Amon.json"))
print('Tables ids:', tables)

cmor.set_table(tables[0])

x, y, lon_coords, lat_coords, lon_vertices, lat_vertices = gen_irreg_grid(
    lon, lat)


myaxes[0] = cmor.axis(table_entry='y',
                      units='m',
                      coord_vals=y)
myaxes[1] = cmor.axis(table_entry='x',
                      units='m',
                      coord_vals=x)

grid_id = cmor.grid(axis_ids=myaxes[:2],
                    latitude=lat_coords,
                    longitude=lon_coords,
                    latitude_vertices=lat_vertices,
                    longitude_vertices=lon_vertices)
print('got grid_id:', grid_id)
myaxes[2] = grid_id

mapnm = 'lambert_conformal_conic'
params = ["standard_parallel1",
          "longitude_of_central_meridian", "latitude_of_projection_origin",
          "false_easting", "false_northing", "standard_parallel2"]
punits = ["", "", "", "", "", ""]
pvalues = [-20., 175., 13., 8., 0., 20.]
cmor.set_grid_mapping(grid_id=myaxes[2],
                      mapping_name=mapnm,
                      parameter_names=params,
                      parameter_values=pvalues,
                      parameter_units=punits)

cmor.set_table(tables[1])
myaxes[3] = cmor.axis(table_entry='time',
                      units='months since 1980')

pass_axes = [myaxes[3], myaxes[2]]
myvars[0] = cmor.variable(table_entry='hfls',
                          units='W m-2',
                          axis_ids=pass_axes,
                          positive='down',
                          original_name='HFLS',
                          history='no history',
                          comment='no future'
                          )
for i in range(ntimes):
    data2d = read_2d_input_files(i, varin2d[0], lat, lon)
    print('writing time: ', i, Time[i], data2d.shape, data2d)
    print(Time[i], bnds_time[2 * i:2 * i + 2])
    cmor.write(myvars[0], data2d, 1, time_vals=Time[i],
               time_bnds=bnds_time[2 * i:2 * i + 2])
cmor.close()



import cmor

import numpy

cmor.setup("Test", netcdf_file_action=cmor.CMOR_REPLACE)

cmor.dataset_json("Test/CMOR_input_example.json")

cmor.load_table("Tables/CMIP6_Omon.json")

nlat = 90
dlat = 180 / nlat
lats = numpy.arange(-90 + dlat / 2., 90, dlat)
blats = numpy.arange(-90, 90 + dlat, dlat)

ilat = cmor.axis(
    table_entry='latitude',
    coord_vals=lats,
    cell_bounds=blats,
    units='degrees_north')

myregions = [
    "atlantic_ocean",
    "indian_pacific_ocean",
    "pacific_ocean",
    "atlantic_arctic_ocean",
    "global_ocean"]
nreg = len(myregions)
ireg = cmor.axis(table_entry='basin', units='1', coord_vals=myregions)
ntime = 12

itim = cmor.axis(
    table_entry='time',
    units='months since 2030',
    interval='1 month',
    coord_vals=numpy.arange(ntime),
    cell_bounds=numpy.arange(
        ntime + 1))


var = cmor.variable(table_entry='htovgyre', units='W',
                    axis_ids=numpy.array([itim, ireg, ilat]))

data = numpy.random.random((ntime, nreg, nlat)) * 3.E14

cmor.write(var, data)
cmor.close()



import os
import json
import cmor
import unittest
from base_CMIP6_CV import BaseCVsTest

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = "TestTables/CMIP7_CV.json"

USER_INPUT = {
    "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
    "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
    "_cmip7_option": 1,
    "_controlled_vocabulary_file": None,
    "activity_id": "CMIP",
    "branch_time_in_child": 30.0,
    "branch_time_in_parent": 10800.0,
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "drs_specs": "MIP-DRS7",
    "experiment_id": "piControl",
    "forcing_index": "f30",
    "grid_label": "gn",
    "initialization_index": "i000001d",
    "institution_id": "PCMDI",
    "license_id": "CC BY 4.0",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "parent_mip_era": "CMIP7",
    "parent_time_units": "days since 1850-01-01",
    "parent_activity_id": "CMIP",
    "parent_source_id": "PCMDI-test-1-0",
    "parent_experiment_id": "piControl",
    "parent_variant_label": "r1i1p1f3",
    "physics_index": "p1",
    "realization_index": "r009",
    "source_id": "PCMDI-test-1-0",
    "tracking_prefix": "hdl:21.14100",
    "host_collection": "CMIP7",
    "frequency": "mon",
    "region": "glb",
    "archive_id": "WCRP"
}


class TestCheckCVStructure(BaseCVsTest):

    def test_check_for_cv_json_key(self):

        test_name = "check_for_cv_json_key"
        cv_path = f"TestTables/CMIP7_CV_{test_name}.json"
        input_path = f"Test/input_{test_name}.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open(CV_PATH, "r") as cv_infile:
            cv = json.load(cv_infile)
            no_cv_key = cv["CV"]
            with open(cv_path, "w") as cv_outfile:
                json.dump(no_cv_key, cv_outfile, sort_keys=True, indent=4)

        with open(input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = cv_path
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        error_flag = cmor.dataset_json(input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        with self.assertRaises(cmor.CMORError):
            cmor.load_table("CMIP7_ocean.json")

        self.assertCV("CV section was not found in table: ")

        os.remove(cv_path)
        os.remove(input_path)

    def test_check_cv_attribute_values(self):

        test_name = "check_cv_attribute_values"
        cv_path = f"TestTables/CMIP7_CV_{test_name}.json"
        input_path = f"Test/input_{test_name}.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open(CV_PATH, "r") as cv_infile:
            cv = json.load(cv_infile)
            cv["CV"]["branding_suffix"] = ["template"]
            cv["CV"]["mip_era"] = {"mip_era": "CMIP7"}
            cv["CV"]["nominal_resolution"] = "10000 km"
            cv["CV"]["frequency"] = ["1hr", "3hr", "day", "mon"]
            cv["CV"]["license"] = "PCMDI"
            with open(cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        with open(input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = cv_path
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        error_flag = cmor.dataset_json(input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        cmor.load_table("CMIP7_ocean.json")

        self.assertCV(
            "must be a string",
            "Warning: Attribute \"branding_suffix\""
        )

        self.assertCV(
            "must be a string or an array",
            "Warning: Attribute \"mip_era\""
        )

        self.assertCV(
            "must be an array or object",
            "Warning: Attribute \"nominal_resolution\""
        )

        self.assertCV(
            "must be an object",
            "Warning: Attribute \"frequency\""
        )

        self.assertCV(
            "must be an array or object",
            "Warning: Attribute \"license\""
        )

        os.remove(cv_path)
        os.remove(input_path)

    def test_check_cv_single_value_pairs(self):

        test_name = "check_cv_single_value_pairs"
        cv_path = f"TestTables/CMIP7_CV_{test_name}.json"
        input_path = f"Test/input_{test_name}.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open(CV_PATH, "r") as cv_infile:
            cv = json.load(cv_infile)
            cv["CV"]["institution_id"]["PCMDI"] = {"LLNL": "Livermore, CA"}
            with open(cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        with open(input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = cv_path
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        error_flag = cmor.dataset_json(input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        cmor.load_table("CMIP7_ocean.json")

        self.assertCV(
            "in attribute \"institution_id\" cannot be an object",
            "Value for \"PCMDI\""
        )

        os.remove(cv_path)
        os.remove(input_path)

    def test_check_cv_array_values(self):

        test_name = "check_cv_array_values"
        cv_path = f"TestTables/CMIP7_CV_{test_name}.json"
        input_path = f"Test/input_{test_name}.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open("TestTables/CMIP7_CV.json", "r") as cv_infile:
            cv = json.load(cv_infile)
            cv["CV"]["source_type"] = [
                "AER",
                "AGCM",
                {"ISM": "ice-sheet model that includes ice-flow"}
                ]
            with open(cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        with open(input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = cv_path
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        error_flag = cmor.dataset_json(input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        cmor.load_table("CMIP7_ocean.json")

        self.assertCV(
            "has elements in its array that are not strings",
            "Attribute \"source_type\""
        )

        os.remove(cv_path)
        os.remove(input_path)

    def test_check_cv_nested_objects(self):

        test_name = "check_cv_nested_objects"
        cv_path = f"TestTables/CMIP7_CV_{test_name}.json"
        input_path = f"Test/input_{test_name}.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open("TestTables/CMIP7_CV.json", "r") as cv_infile:
            cv = json.load(cv_infile)
            cv["CV"]["nested_attr"] = {}
            cv["CV"]["nested_attr"]["nested_value"] = {
                "sub_attr1": "value",
                "sub_attr2": ["1", "2"],
                "sub_attr3": ["1", ["2"], "3"],
                "sub_attr4": {"key": "value"},
                "sub_attr5": {"key": ["value"]}
            }

            with open(cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        with open(input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = cv_path
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        error_flag = cmor.dataset_json(input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        cmor.load_table("CMIP7_ocean.json")

        self.assertCV(
            "has elements in its array that are not strings",
            "Attribute \"sub_attr3\""
        )

        self.assertCV(
            "in attribute \"sub_attr5\" cannot be an array",
            "Value for \"key\""
        )

        os.remove(cv_path)
        os.remove(input_path)


if __name__ == '__main__':
    unittest.main()



import os
import json
import cmor
import unittest
import numpy

from netCDF4 import Dataset
import pyfive

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = "TestTables/CMIP7_CV.json"

USER_INPUT = {
    "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
    "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
    "_cmip7_option": 1,
    "_controlled_vocabulary_file": CV_PATH,
    "activity_id": "CMIP",
    "branch_time_in_child": 30.0,
    "branch_time_in_parent": 10800.0,
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "drs_specs": "MIP-DRS7",
    "experiment_id": "piControl",
    "forcing_index": "f30",
    "grid_label": "gn",
    "initialization_index": "i000001d",
    "institution_id": "PCMDI",
    "license_id": "CC BY 4.0",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "parent_mip_era": "CMIP7",
    "parent_time_units": "days since 1850-01-01",
    "parent_activity_id": "CMIP",
    "parent_source_id": "PCMDI-test-1-0",
    "parent_experiment_id": "piControl",
    "parent_variant_label": "r1i1p1f3",
    "physics_index": "p1",
    "realization_index": "r009",
    "source_id": "PCMDI-test-1-0",
    "tracking_prefix": "hdl:21.14100",
    "host_collection": "CMIP7",
    "frequency": "day",
    "region": "glb",
    "archive_id": "WCRP",
    "output_path_template": "<activity_id><source_id><experiment_id><member_id><variable_id><branding_suffix><grid_label>",
    "output_file_template": "<variable_id><branding_suffix><frequency><region><grid_label><source_id><experiment_id><variant_label>",
}


class TestChunking(unittest.TestCase):

    def assertCMIP7RepackChunking(self, filename, msg=None):
        BYTES_4MiB = 4 * 2**20

        """ Chunking checks from cmip7_repack """
        f = pyfive.File(filename)
        if "time" in f:
            # Check for the time coordinates variable having one chunk
            t = f["time"]
            chunks = t.chunks
            if chunks is not None and t.id.get_num_chunks() > 1:
                # At least two chunks
                message = self._formatMessage(msg,
                    f"FAIL: File {filename!r} time coordinates variable "
                    f"'time' has {t.id.get_num_chunks()} chunks "
                    "(expected 1 chunk or contiguous)"
                )
                raise self.failureException(message)

            # Check for the time bounds variable having one chunk
            if "bounds" in t.attrs:
                bounds = str(numpy.array(t.attrs["bounds"]).astype("U"))
                if bounds in f:
                    b = f[bounds]
                    chunks = b.chunks
                    if chunks is not None and b.id.get_num_chunks() > 1:
                        # At least two chunks
                        message = self._formatMessage(msg,
                            f"FAIL: File {filename!r} time bounds variable "
                            f"{bounds!r} has {b.id.get_num_chunks()} chunks "
                            "(expected 1 chunk or contiguous)"
                        )
                        raise self.failureException(message)

        # Check for the data variable having one chunks of at least ~4MiB
        if "variable_id" in f.attrs:
            variable_id = str(numpy.array(f.attrs["variable_id"]).astype("U"))
            if variable_id in f:
                d = f[variable_id]
                if chunks is not None and d.id.get_num_chunks() > 1:
                    # At least two chunks
                    chunks = d.chunks
                    wordsize = d.dtype.itemsize
                    chunksize = numpy.prod(chunks) * wordsize

                    lee_way = 0
                    if len(chunks) > 1:
                        lee_way = numpy.prod(chunks[1:]) * wordsize

                    if chunksize + lee_way < BYTES_4MiB:
                        message = self._formatMessage(msg,
                            f"FAIL: File {filename!r} data variable "
                            f"{variable_id!r} has uncompressed chunk size "
                            f"{chunksize} B (expected at least "
                            f"{BYTES_4MiB - lee_way} B or 1 chunk "
                            "or contiguous)"
                        )
                        raise self.failureException(message)
        
    def setUp(self):
        """
        Write out a simple file using CMOR
        """
        # Set up CMOR
        cmor.setup(inpath=CMIP7_TABLES_PATH, netcdf_file_action=cmor.CMOR_REPLACE)

        # Define dataset using USER_INPUT
        with open("Test/input_cmip7.json", "w") as input_file_handle:
            json.dump(USER_INPUT, input_file_handle, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json("Test/input_cmip7.json")
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")
        
        self.cmor_filepath = ""

    def tearDown(self):
        if os.path.isfile(self.cmor_filepath):
            os.remove(self.cmor_filepath)

    def generate_file(self, num_times, num_lat, num_lon):

        data = [27] * (num_times * num_lat * num_lon)
        pr = numpy.array(data)
        pr.shape = (num_times, num_lat, num_lon)
        lat_bnds = numpy.linspace(-90., 90., num_lat + 1)
        lat = (lat_bnds[:-1] + lat_bnds[1:]) / 2
        lon_bnds = numpy.linspace(0., 180., num_lon + 1)
        lon = (lon_bnds[:-1] + lon_bnds[1:]) / 2

        time = numpy.arange(num_times) + 0.5
        time_bnds = numpy.arange(num_times + 1)

        cmor.load_table("CMIP7_atmos.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             coord_vals=time,
                             cell_bounds=time_bnds,
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmorpr = cmor.variable("pr_tavg-u-hxy-u", "kg m-2 s-1", axes)
        self.assertEqual(cmor.write(cmorpr, pr), 0)
        self.cmor_filepath = cmor.close(cmorpr, file_name=True)
        self.assertEqual(cmor.close(), 0)

    def test_few_small_timesteps(self):
        num_times = 4
        num_lat = 100
        num_lon = 100
        self.generate_file(num_times, num_lat, num_lon)

        self.assertCMIP7RepackChunking(self.cmor_filepath)

    def test_many_small_timesteps(self):
        num_times = 200
        num_lat = 100
        num_lon = 100
        self.generate_file(num_times, num_lat, num_lon)

        self.assertCMIP7RepackChunking(self.cmor_filepath)

    def test_large_timesteps(self):
        num_times = 10
        num_lat = 1000
        num_lon = 1000
        self.generate_file(num_times, num_lat, num_lon)

        self.assertCMIP7RepackChunking(self.cmor_filepath)

    def test_writing_one_timestep_at_a_time(self):
        num_times = 4
        num_lat = 100
        num_lon = 100

        data = [27] * (num_times * num_lat * num_lon)
        pr = numpy.array(data)
        pr.shape = (num_times, num_lat, num_lon)
        lat_bnds = numpy.linspace(-90., 90., num_lat + 1)
        lat = (lat_bnds[:-1] + lat_bnds[1:]) / 2
        lon_bnds = numpy.linspace(0., 180., num_lon + 1)
        lon = (lon_bnds[:-1] + lon_bnds[1:]) / 2

        time = numpy.arange(num_times) + 0.5
        time_bnds = numpy.arange(num_times + 1)

        cmor.load_table("CMIP7_atmos.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmorpr = cmor.variable("pr_tavg-u-hxy-u", "kg m-2 s-1", axes)
        for i in range(num_times):
            self.assertEqual(cmor.write(cmorpr, pr[i,:,:], time_vals=time[i], time_bnds=time_bnds[i:i+2]), 0)
        self.cmor_filepath = cmor.close(cmorpr, file_name=True)
        self.assertEqual(cmor.close(), 0)

        with Dataset(self.cmor_filepath, 'r') as dataset:
            pr_chunks = dataset.variables['pr'].chunking()
            self.assertEqual(pr_chunks, [1, num_lat, num_lon])
            time_chunks = dataset.variables['time'].chunking()
            self.assertEqual(time_chunks, [512])
            time_bnds_chunks = dataset.variables['time_bnds'].chunking()
            self.assertEqual(time_bnds_chunks, [512, 2])

    def test_user_defined_variable_chunking(self):
        num_times = 40
        num_lat = 100
        num_lon = 100
        chunking_dims = [8, 20, 20]

        data = [27] * (num_times * num_lat * num_lon)
        pr = numpy.array(data)
        pr.shape = (num_times, num_lat, num_lon)
        lat_bnds = numpy.linspace(-90., 90., num_lat + 1)
        lat = (lat_bnds[:-1] + lat_bnds[1:]) / 2
        lon_bnds = numpy.linspace(0., 180., num_lon + 1)
        lon = (lon_bnds[:-1] + lon_bnds[1:]) / 2

        time = numpy.arange(num_times) + 0.5
        time_bnds = numpy.arange(num_times + 1)

        cmor.load_table("CMIP7_atmos.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmorpr = cmor.variable("pr_tavg-u-hxy-u", "kg m-2 s-1", axes)
        cmor.set_chunking(cmorpr, chunking_dims)
        for i in range(num_times):
            self.assertEqual(cmor.write(cmorpr, pr[i,:,:], time_vals=time[i], time_bnds=time_bnds[i:i+2]), 0)
        self.cmor_filepath = cmor.close(cmorpr, file_name=True)
        self.assertEqual(cmor.close(), 0)

        with Dataset(self.cmor_filepath, 'r') as dataset:
            pr_chunks = dataset.variables['pr'].chunking()
            self.assertEqual(pr_chunks, chunking_dims)
            time_chunks = dataset.variables['time'].chunking()
            self.assertEqual(time_chunks, [512])
            time_bnds_chunks = dataset.variables['time_bnds'].chunking()
            self.assertEqual(time_bnds_chunks, [512, 2])


if __name__ == '__main__':
    unittest.main()



import json
import cmor
import unittest
import os
import numpy

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = "TestTables/CMIP7_CV.json"

USER_INPUT = {
    "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
    "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
    "_cmip7_option": 1,
    "_controlled_vocabulary_file": None,
    "activity_id": "CMIP",
    "branch_time_in_child": 30.0,
    "branch_time_in_parent": 10800.0,
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "drs_specs": "MIP-DRS7",
    "experiment_id": "piControl",
    "forcing_index": "f30",
    "grid_label": "gn",
    "initialization_index": "i000001d",
    "institution_id": "PCMDI",
    "license_id": "CC BY 4.0",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "parent_mip_era": "CMIP7",
    "parent_time_units": "days since 1850-01-01",
    "parent_activity_id": "CMIP",
    "parent_source_id": "PCMDI-test-1-0",
    "parent_experiment_id": "piControl",
    "parent_variant_label": "r1i1p1f3",
    "physics_index": "p1",
    "realization_index": "r009",
    "source_id": "PCMDI-test-1-0",
    "tracking_prefix": "hdl:21.14100",
    "host_collection": "CMIP7",
    "frequency": "mon",
    "region": "glb",
    "archive_id": "WCRP"
}


class TestPathAndFileTemplates(unittest.TestCase):

    def gen_cmor_file(self):
        tos = numpy.array([27, 27, 27, 27,
                           27, 27, 27, 27,
                           27, 27, 27, 27,
                           27, 27, 27, 27,
                           27, 27, 27, 27,
                           27, 27, 27, 27
                           ])
        tos.shape = (2, 3, 4)
        lat = numpy.array([10, 20, 30])
        lat_bnds = numpy.array([5, 15, 25, 35])
        lon = numpy.array([0, 90, 180, 270])
        lon_bnds = numpy.array([-45, 45,
                                135,
                                225,
                                315
                                ])
        time = numpy.array([15.5, 45])
        time_bnds = numpy.array([0, 31, 60])
        cmor.load_table("CMIP7_ocean.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             coord_vals=time,
                             cell_bounds=time_bnds,
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmortos = cmor.variable("tos_tavg-u-hxy-sea", "degC", axes)
        self.assertEqual(cmor.write(cmortos, tos), 0)
        version = cmor.get_cur_dataset_attribute('_version')
        filepath = cmor.close(cmortos, file_name=True)

        # Attributes for reconstructing file path
        attrs = USER_INPUT.copy()
        variant_label = ('{realization_index}{initialization_index}'
                         '{physics_index}{forcing_index}').format(**attrs)
        attrs['mip_era'] = 'CMIP7'
        attrs['table_id'] = 'ocean'
        attrs['variable_id'] = 'tos'
        attrs['version'] = version
        attrs['variant_label'] = variant_label
        attrs['branding_suffix'] = 'tavg-u-hxy-sea'

        return (filepath, attrs)

    def test_default_path_and_file_templates(self):

        test_name = "default_path_and_file_templates"
        test_cv_path = f"TestTables/CMIP7_CV_{test_name}.json"

        # Set up CMOR
        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE)

        # Remove "DRS" section from CV to use default
        # directory path and file name templates in CMOR
        with open(CV_PATH, "r") as cv_infile:
            cv = json.load(cv_infile)
            cv["CV"].pop("DRS")
            with open(test_cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        # Define dataset using USER_INPUT
        with open("Test/input_drs.json", "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = test_cv_path
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json("Test/input_drs.json")
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        filepath, attrs = self.gen_cmor_file()

        predicted_path = ('./{mip_era}/{activity_id}/{institution_id}/'
                          '{source_id}/{experiment_id}/{variant_label}/'
                          '{table_id}/{variable_id}/{grid_label}/{version}'
                          ).format(**attrs)
        predicted_file = ('{variable_id}_{table_id}_{source_id}_'
                          '{experiment_id}_{variant_label}_{grid_label}_'
                          '201801-201802.nc').format(**attrs)
        predicted_filepath = os.path.join(predicted_path, predicted_file)
        self.assertEqual(filepath, predicted_filepath)

        os.remove(filepath)
        os.remove(test_cv_path)

    def test_cv_path_and_file_templates(self):

        test_name = "cv_path_and_file_templates"
        test_cv_path = f"TestTables/CMIP7_CV_{test_name}.json"

        # Set up CMOR
        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE)

        # Use "DRS" section from CV
        drs = {}
        with open(CV_PATH, "r") as cv_infile:
            cv = json.load(cv_infile)
            drs = cv["CV"]["DRS"]
            with open(test_cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        # Define dataset using USER_INPUT
        with open("Test/input_drs.json", "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = test_cv_path
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json("Test/input_drs.json")
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        filepath, attrs = self.gen_cmor_file()

        predicted_path_template = drs["directory_path_template"].replace("/","")
        predicted_path_template = predicted_path_template.replace("<","{").replace(">","}")
        predicted_path_template = predicted_path_template.replace("}{","}/{")

        predicted_file_template = drs["filename_template"]
        predicted_file_template = predicted_file_template.replace("<","{").replace(">","}")
        predicted_file_template = predicted_file_template.replace("}{","}_{")

        predicted_path = f'./{predicted_path_template.format(**attrs)}'
        predicted_file = f'{predicted_file_template.format(**attrs)}_201801-201802.nc'
        predicted_filepath = os.path.join(predicted_path, predicted_file)

        self.assertEqual(filepath.replace('//','/'), predicted_filepath)

        os.remove(filepath)
        os.remove(test_cv_path)

    def test_user_input_path_and_file_templates(self):

        test_name = "user_input_path_and_file_templates"
        test_cv_path = f"TestTables/CMIP7_CV_{test_name}.json"

        # Set up CMOR
        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE)

        # Use "DRS" section from CV
        with open(CV_PATH, "r") as cv_infile:
            cv = json.load(cv_infile)
            with open(test_cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        # Use directory path and file name templates from user input
        with open("Test/input_drs.json", "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["_controlled_vocabulary_file"] = test_cv_path
            user_input["output_path_template"] = (
                "<activity_id><source_id><experiment_id>"
                "<frequency><variable_id><branding_suffix>"
            )
            user_input["output_file_template"] = (
                "<region><grid_label><variant_label><version>"
            )
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json("Test/input_drs.json")
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        filepath, attrs = self.gen_cmor_file()

        predicted_path = ('./{activity_id}/{source_id}/{experiment_id}/'
                          '{frequency}/{variable_id}/{branding_suffix}/'
                          ).format(**attrs)
        predicted_file = ('{region}_{grid_label}_{variant_label}_{version}'
                          '_201801-201802.nc').format(**attrs)
        predicted_filepath = os.path.join(predicted_path, predicted_file)
        self.assertEqual(filepath, predicted_filepath)

        os.remove(filepath)
        os.remove(test_cv_path)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
'''
Test program for creating a sample CFMIP site axis.

This program assumes that we want the output netcdf header to look something like this (in CDL):

dimensions:
   site = 119;
   lev = 38;
   time = UNLIMITED;
   bnds = 2;
variables:
   float cl(time, site, lev);
      cl:coordinates = "lat lon";
      ...
   double time(time);
      ...
   int site(site);    // integer id values for sites
      ...
   float lat(site);   // latitude values for sites
      ...
   float lon(site);   // longitude values for sites
      ...
   float orog(site);  // orography values for sites
      ...
   float lev(lev);    // hybrid_height
      lev:formula = "z(k,m) = a(k) + b(k)*orog(m)";   // m = site index
      lev:formula_terms = "a: lev b: b orog: orog";
      ...
'''
import cmor
import numpy

MIP_TABLE_DIR = 'Tables'   # set according to your MIP table location

#-------------------------------------------------------------------------


def setup_cmor():
    #-------------------------------------------------------------------------
    # Initialise CMOR library
    cmor.setup(inpath=MIP_TABLE_DIR, netcdf_file_action=cmor.CMOR_REPLACE_3,
               set_verbosity=cmor.CMOR_NORMAL, create_subdirectories=0)

    # Create CMOR dataset
    cmor.dataset_json("Test/CMOR_input_example.json")


#-------------------------------------------------------------------------
if __name__ == '__main__':
    #-------------------------------------------------------------------------

    # Initialise CMOR dataset and table
    setup_cmor()

    # Set dummy site lats and longs.
    site_lats = numpy.array([-90.0, 0.0, 90.0], dtype=numpy.float32)
    site_lons = numpy.array([0.0, 0.0, 0.0], dtype=numpy.float32)

    # Create CMOR axes and grids
    table_id = cmor.load_table('CMIP6_CFsubhr.json')
    # , length=1, interval='30 minutes')
    taxis_id = cmor.axis('time1', units='days since 2000-01-01 00:00:00')
    print('ok: created time axis')

    saxis_id = cmor.axis('site', units='1', coord_vals=[1, 2, 3])
    print('ok: created site axis', saxis_id)

    zaxis_id = cmor.axis(
        'hybrid_height',
        units='m',
        coord_vals=[1.0],
        cell_bounds=[
            0.0,
            2.0])
    print('ok: created height axis', zaxis_id)

    # Create zfactors for b and orog for hybrid height axis.
    # Where do these get used, if anywhere?
    bfact_id = cmor.zfactor(zaxis_id, 'b', '1', [zaxis_id], 'd', zfactor_values=[1.0],
                            zfactor_bounds=[0.0, 2.0])
    print('ok: created b zfactors')

    # Create grid object to link site-dimensioned variables to (lat,long).
    # Need to make CMIP6_grids the current MIP table for this to work.
    table_id = cmor.load_table('CMIP6_grids.json')
    gaxis_id = cmor.grid([saxis_id], site_lats, site_lons)
    print('ok: created site grid')

    # Create CMOR variable for cloud area fraction: MIP name = 'cl', STASH =
    # m01s02i261*100
    table_id = cmor.load_table('CMIP6_CFsubhr.json')
    var_id = cmor.variable('cl', '%', [taxis_id, gaxis_id, zaxis_id], data_type='f',
                           missing_value=-99.0, original_name='STASH m01s02i261*100')
    print('ok: created variable for "cl"')

    ofact_id = cmor.zfactor(zaxis_id, 'orog', 'm', [gaxis_id], 'd',
                            zfactor_values=[123.0, 123.0, 123.0])
    print('ok: created orog zfactors')
    # Write some data to this variable. First convert raw data to numpy arrays.
    shape = (1, 3, 1)
    data = numpy.array([10, 20, 30], dtype=numpy.float32)
    data = data.reshape(shape)
    cmor.write(var_id, data, time_vals=[1.0])
    print('ok: wrote variable data')

    # Close CMOR.
    cmor.close()



import cmor
import numpy

breq = "100000. 80000. 80000. 68000. 68000. 56000. 56000. 44000. 44000. 31000. 31000. 18000. 18000.  0.".split()

bnds_req = []
for b in breq:
    bnds_req.append(float(b))

bnds_req = numpy.array(bnds_req)
bnds_req.shape = (7, 2)

print(bnds_req[-2], bnds_req.shape)

levs = []

for b in bnds_req:
    levs.append((b[0] + b[1]) / 2.)

levs = numpy.array(levs)

print(levs)

cmor.setup(inpath="Tables",
           set_verbosity=cmor.CMOR_NORMAL,
           netcdf_file_action=cmor.CMOR_REPLACE,
           logfile=None)

cmor.dataset_json("Test/common_user_inputNOBOUNDS.json")

cmor.load_table("TestTables/python_test_table_A")

nlat = 90
dlat = 180 / nlat
nlon = 180
dlon = 360. / nlon

lats = numpy.arange(-90 + dlat / 2., 90, dlat)
lons = numpy.arange(0, 360., dlon)

ntime = 12

data = numpy.random.random((ntime, 7, nlat, nlon)) + 280.

itim = cmor.axis(
    table_entry='time',
    coord_vals=numpy.arange(
        0,
        ntime,
        1),
    units='month since 2008')
ilat = cmor.axis(
    table_entry='latitude',
    coord_vals=lats,
    units='degrees_north')
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=lons,
    units='degrees_east')
print('so far', itim, ilat, ilon)
print(bnds_req)
ilev = cmor.axis(
    table_entry="pressure2",
    coord_vals=levs,
    cell_bounds=bnds_req,
    units="Pa")

iv = cmor.variable(
    table_entry='ta', axis_ids=numpy.array(
        (itim, ilev, ilat, ilon)), units='K')

cmor.write(iv, data)



import cmor
import numpy

error_flag = cmor.setup(inpath='Test', netcdf_file_action=cmor.CMOR_REPLACE)

error_flag = cmor.dataset_json("Test/CMOR_input_example.json")

# creates 10 degree grid
nlat = 18
nlon = 36
alats = numpy.arange(180) - 89.5
bnds_lat = numpy.arange(181) - 90.
alons = numpy.arange(360) + .5
bnds_lon = numpy.arange(361)
cmor.load_table("Tables/CMIP6_Amon.json")
ilat = cmor.axis(
    table_entry='latitude',
    units='degrees_north',
    length=nlat,
    coord_vals=alats,
    cell_bounds=bnds_lat)

ilon = cmor.axis(
    table_entry='longitude',
    length=nlon,
    units='degrees_east',
    coord_vals=alons,
    cell_bounds=bnds_lon)

ntimes = 600
plevs = numpy.array([100000., 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
                     20000, 15000, 10000, 7000, 5000, 3000, 2000, 1000, 999, 998, 997, 996,
                     995, 994, 993, 992, 500, 100])


itim = cmor.axis(
    table_entry='time',
    units='months since 2030-1-1',
    length=ntimes,
    interval='1 month')

ilev = cmor.axis(
    table_entry='plev19',
    units='Pa',
    coord_vals=plevs,
    cell_bounds=None)


for it in range(ntimes):

    var3d_ids = cmor.variable(
        table_entry='ta',
        units='K',
        comment='My wise comments here',
        axis_ids=numpy.array((ilev, ilon, ilat, itim)),
        missing_value=numpy.array([1.0e28, ], dtype=numpy.float32)[0],
        original_name='cloud')

    time = numpy.array((it))
    bnds_time = numpy.array((it, it + 1))
    data3d = numpy.random.random((len(plevs), nlon, nlat)) * 30. + 265.
    data3d = data3d.astype('f')
    error_flag = cmor.write(
        var_id=var3d_ids,
        data=data3d,
        ntimes_passed=1,
        time_vals=time,
        time_bnds=bnds_time)

    cmor.close(var3d_ids)


error_flag = cmor.close()



from test_python_common import *  # common subroutines

import cmor._cmor
import os

pth = os.path.split(os.path.realpath(os.curdir))


myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')


cmor.setup(
    inpath="Tables",
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE,
    exit_control=cmor.CMOR_EXIT_ON_MAJOR)
cmor.dataset_json("Test/CMOR_input_example.json")

tables = []
a = cmor.load_table("Tables/CMIP6_grids.json")
tables.append(a)
tables.append(cmor.load_table("Tables/CMIP6_Omon.json"))
print('Tables ids:', tables)

cmor.set_table(tables[0])

x, y, lon_coords, lat_coords, lon_vertices, lat_vertices = gen_irreg_grid(
    lon, lat)


myaxes[0] = cmor.axis(table_entry='y',
                      units='m',
                      coord_vals=y)
myaxes[1] = cmor.axis(table_entry='x',
                      units='m',
                      coord_vals=x)

grid_id = cmor.grid(axis_ids=myaxes[:2],
                    latitude=lat_coords,
                    longitude=lon_coords,
                    latitude_vertices=lat_vertices,
                    longitude_vertices=lon_vertices)
print('got grid_id:', grid_id)
myaxes[2] = grid_id

## mapnm = 'lambert_conformal_conic'
# params = [ "standard_parallel1",
# "longitude_of_central_meridian","latitude_of_projection_origin",
# "false_easting","false_northing","standard_parallel2" ]
## punits = ["","","","","","" ]
## pvalues = [-20.,175.,13.,8.,0.,20. ]
# cmor.set_grid_mapping(grid_id=myaxes[2],
##                       mapping_name = mapnm,
##                       parameter_names = params,
##                       parameter_values = pvalues,
# parameter_units = punits)

cmor.set_table(tables[1])
myaxes[3] = cmor.axis(table_entry='time',
                      units='months since 1980')
myaxes[4] = cmor.axis(table_entry='oline',
                      units='',
                      coord_vals="""barents_opening bering_strait canadian_archipelago denmark_strait drake_passage english_channel pacific_equatorial_undercurrent faroe_scotland_channel florida_bahamas_strait fram_strait iceland_faroe_channel indonesian_thoughflow mozambique_channel taiwan_luzon_straits windward_passage""".split())

pass_axes = [myaxes[3], myaxes[4]]

print('ok going to cmorvar')
myvars[0] = cmor.variable(table_entry='mfo',
                          units='kg s-1',
                          axis_ids=pass_axes,
                          history='no history',
                          comment='no future'
                          )
for i in range(ntimes):
    data2d = numpy.random.random((1, 15))
    print('writing time: ', i, data2d.shape, data2d)
    print(Time[i], bnds_time[2 * i:2 * i + 2])
    cmor.write(myvars[0], data2d, 1, time_vals=Time[i],
               time_bnds=bnds_time[2 * i:2 * i + 2])
cmor.close()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.
import cmor
import base_CMIP6_CV
import sys
import os
import tempfile
import signal
import unittest

def run():
    unittest.main()

class TestdirectoryMethods(base_CMIP6_CV.BaseCVsTest):

    def test_Directory(self):

        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        try:
            cmor.dataset_json("Test/baddirectory.json")
        except BaseException:
            pass
        self.assertCV("unable to create this directory")

if __name__ == '__main__':
    run()



import numpy
import cmor
import os

specs = {"CLOUD": {"convert": [2.0, 0.], "units": "%", "entry": "cl", "positive": ""},
         "U": {"convert": [1., -40.], "units": "m s-1", "entry": "ua", "positive": ""},
         "T": {"convert": [1., 220], "units": "K", "entry": "ta", "positive": ""},
         "LATENT": {"convert": [4., 0.], "units": "W m-2", "entry": "hfls", "positive": "down"},
         "TSURF": {"convert": [1., -15.], "units": "Celsius", "entry": "tas", "positive": ""},
         "PSURF": {"convert": [3., 900.], "units": "hPa", "entry": "ps", "positive": ""},
         "SOIL_WET": {"convert": [2., 0.], "units": "kg m-2", "entry": "mrsos", "positive": ""},
         "MC": {"convert": [2., 0.], "units": "kg m-2 s-1", "entry": "mc", "positive": "up"},
         "BS": {"convert": [2., 0.], "units": "m-1 sr-1", "entry": "bs550aer", "positive": ""},
         }


def read_coords(nlats, nlons):
    alons = (360.*numpy.arange(nlons)/nlons+.5) - 180.
    blons = numpy.zeros((nlons, 2), dtype=float)
    blons[:, 0] = 360.*numpy.arange(nlons)/nlons - 180.
    blons[:, 1] = 360.*numpy.arange(1, nlons+1)/nlons - 180.

    alats = (180.*numpy.arange(nlats)/nlats+.5) - 90.
    blats = numpy.zeros((nlats, 2), dtype=float)
    blats[:, 0] = 180.*numpy.arange(nlats)/nlats - 90.
    blats[:, 1] = 180.*numpy.arange(1, nlats+1)/nlats - 90.

    levs = numpy.array([1., 5., 10, 20, 30, 50, 70, 100, 150,
                        200, 250, 300, 400, 500, 600, 700, 850, 925, 1000])

    return alats, blats, alons, blons, levs


def read_time(index, refyear=2015., monthdays=30., yeardays=360.):
    time0 = (refyear - 1850.) * yeardays
    time = time0 + (index + 0.5) * monthdays
    b1 = time0 + (index) * monthdays
    b2 = time0 + (index+1) * monthdays
    return numpy.array([time]), numpy.array([b1, b2])


def read_3d_input_files(index, varname, shape):

    print("3d shape", shape)
    field = numpy.zeros(shape, dtype=numpy.float32)
    factor, offset = specs[varname]["convert"]
    for i in range(field.shape[2]):  # lon
        for j in range(field.shape[1]):  # lat
            for k in range(field.shape[0]):  # levs
                if i == 0:
                    field[k, j, 0] = ((k)*3 + (j)*16 + index) * \
                        factor + offset + 0.225
                else:
                    field[k, j, i] = ((k)*3 + (j)*16 + index +
                                      4*i-2)*factor + offset + 0.225
    return field


def read_2d_input_files(index, varname, shape):
    sh = list(shape)
    sh.insert(0, 1)
    return read_3d_input_files(index, varname, sh)[0]


# PARAMETERS

ntimes = 2
lon = 4
lat = 3
plev = 19
lev = 5

p0 = 1.e3
a_coeff = [0.1, 0.2, 0.3, 0.22, 0.1]
b_coeff = [0.0, 0.1, 0.2, 0.5, 0.8]

a_coeff_bnds = [0., .15, .25, .25, .16, 0.]
b_coeff_bnds = [0., .05, .15, .35, .65, 1.]


zlevs = numpy.array([.1, .3, .55, .7, .9])
zlev_bnds = numpy.array([0., .2, .42, .62, .8, 1.])

alats, bnds_lat, alons, bnds_lon, plevs = read_coords(lat, lon)

def init_cmor(pth, dataset_json):
    ierr = cmor.setup(inpath=os.path.join(pth, "Test"),
                    netcdf_file_action=cmor.CMOR_REPLACE) #, logfile="CMOR.log")
    ierr = cmor.dataset_json(os.path.join(pth, "Test", dataset_json))

def read_cmor_time_lat_lon():
    ilat = cmor.axis(
        table_entry='latitude',
        units='degrees_north',
        length=lat,
        coord_vals=alats,
        cell_bounds=bnds_lat)
    print("ILAT:", ilat)
    print(lon, alons, bnds_lon)
    ilon = cmor.axis(
        table_entry='longitude',
        coord_vals=alons,
        units='degrees_east',
        cell_bounds=bnds_lon)

    print("ILON:", ilon)


    itim = cmor.axis(
        table_entry="time",
        units="days since 1850",
        length=ntimes)
    return itim, ilat, ilon

def read_cmor_time1_lat_lon():
    ilat = cmor.axis(
        table_entry='latitude',
        units='degrees_north',
        length=lat,
        coord_vals=alats,
        cell_bounds=bnds_lat)
    print("ILAT:", ilat)
    print(lon, alons, bnds_lon)
    ilon = cmor.axis(
        table_entry='longitude',
        coord_vals=alons,
        units='degrees_east',
        cell_bounds=bnds_lon)

    print("ILON:", ilon)


    itim = cmor.axis(
        table_entry="time1",
        units="days since 1850",
        length=ntimes)
    return itim, ilat, ilon


#!/usr/bin/env python
import cmor
import cdms2
import numpy
import os

# Create some empty arrays
# -------------------------
myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')

# Initialize CMOR
# -------------------
cmor.setup(
    inpath="Tables",
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE_4)
cmor.dataset_json("Test/CMOR_input_example.json")

tables = []
grid_table = cmor.load_table("CMIP6_grids.json")
tables.append(grid_table)

t = 'CMIP6_fx.json'
te = 'orog'
u = 'm'

tables.append(cmor.load_table("%s" % t))
print('Tables ids:', tables)

cmor.set_table(tables[0])

f = cdms2.open("~/Downloads/orog_GIS_LGGE_ELMER2_asmb.nc")
fg = cdms2.open("~/Downloads/Greenland_5km_v1.1.nc")
x = f['x'][:]
y = f['y'][:]
lon_coords = fg['lon'][0, :]
lat_coords = fg['lat'][0, :]


myaxes[0] = cmor.axis(table_entry='y',
                      units='m',
                      coord_vals=y)
myaxes[1] = cmor.axis(table_entry='x',
                      units='m',
                      coord_vals=x)

grid_id = cmor.grid(axis_ids=myaxes[:2],
                    latitude=lat_coords,
                    longitude=lon_coords)

print('got grid_id:', grid_id)
myaxes[2] = grid_id

mapnm = 'polar_stereographic'
params = ["standard_parallel",
          "latitude_of_projection_origin",
          "false_easting",
          "false_northing",
          "straight_vertical_longitude_from_pole",
          "scale_factor_at_projection_origin"]

punits = ["", "", "", "", "", ""]
pvalues = [71., 90., 0., 0., -39., 1.]
cmor.set_grid_mapping(grid_id=myaxes[2],
                      mapping_name=mapnm,
                      parameter_names=params,
                      parameter_values=pvalues,
                      parameter_units=punits)

cmor.set_table(tables[1])
pass_axes = [myaxes[2]]

data = f['orog'][0, :]
myvars[0] = cmor.variable(table_entry=te,
                          units=u,
                          axis_ids=pass_axes,
                          missing_value=data.missing,
                          history='',
                          comment=''
                          )
cmor.write(myvars[0], data)

cmor.close()



import cmor
import numpy


vars = {
    'hfls': [
        'W.m-2',
        25.,
        40.],
    'tas': [
        'K',
        25,
        268.15],
    'clt': [
        '%',
        10000.,
        0.],
    'ta': [
        'K',
        25,
        273.15]}


nlat = 90
dlat = 180 / nlat
nlon = 180
dlon = 360. / nlon
nlev = 19
ntimes = 12

lats = numpy.arange(-90 + dlat / 2., 90, dlat)
blats = numpy.arange(-90, 90 + dlat, dlat)
lons = numpy.arange(0 + dlon / 2., 360., dlon)
blons = numpy.arange(0, 360. + dlon, dlon)


tvars = ['hfls', 'tas', 'clt', 'ta']

cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
cmor.dataset_json("Test/CMOR_input_example.json")
table = 'CMIP6_Amon.json'
cmor.load_table(table)

for var in tvars:
    ilat = cmor.axis(
        table_entry='latitude',
        coord_vals=lats,
        cell_bounds=blats,
        units='degrees_north')
    ilon = cmor.axis(
        table_entry='longitude',
        coord_vals=lons,
        cell_bounds=blons,
        units='degrees_east')
    itim = cmor.axis(
        table_entry='time', coord_vals=numpy.arange(
            ntimes, dtype=float), cell_bounds=numpy.arange(
            ntimes + 1, dtype=float), units='months since 2000')
    ilev = cmor.axis(table_entry='plev19',
                     coord_vals=numpy.array([1000.,
                                             925,
                                             850,
                                             700,
                                             600,
                                             500,
                                             400,
                                             300,
                                             250,
                                             200,
                                             150,
                                             100,
                                             70,
                                             50,
                                             30,
                                             20,
                                             10,
                                             5,
                                             1]),
                     units='hPa')

    if var != 'ta':
        axes = [itim, ilat, ilon]
        data = numpy.random.random(
            (ntimes, nlat, nlon)) * vars[var][1] + vars[var][2]
    else:
        axes = [itim, ilev, ilat, ilon]
        data = numpy.random.random(
            (ntimes, nlev, nlat, nlon)) * vars[var][1] + vars[var][2]

    kw = {}
    if var in ['hfss', 'hfls']:
        kw['positive'] = 'up'
    var = cmor.variable(
        table_entry=var,
        units=vars[var][0],
        axis_ids=axes,
        **kw)

    cmor.write(var, data)
    path = cmor.close(var, file_name=True)
    print('Saved in:', path)

cmor.close()


print('hello')



import cmor
import numpy


def main():

    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE)
    cmor.dataset_json("Test/CMOR_input_example.json")

    table = 'CMIP6_Omon.json'
    cmor.load_table(table)
    axes = [{'table_entry': 'time',
             'units': 'days since 1850-01-01 00:00:00',
             'coord_vals': [15.5, 45, ],
             'cell_bounds':[[0, 31], [31, 62]]
             },
            {'table_entry': 'depth_coord_half',
             'units': 'm',
             'coord_vals': [5000., 3000., 2000., 1000.],
             'cell_bounds': [5000., 3000., 2000., 1000., 0]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            ]

    axis_ids = list()
    for axis in axes:
        print('doing: {}'.format(axis))
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    for var, units, value in (('zhalfo', 'm', 274.),):
        values = numpy.ones(map(lambda x: len(x["coord_vals"]), axes)) * value
        values = values.astype("f")
        varid = cmor.variable(var,
                              units,
                              axis_ids,
                              history='variable history',
                              missing_value=-99
                              )
        cmor.write(varid, values)

    cmor.close()


if __name__ == '__main__':

    main()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import os
import sys
import tempfile
import base_CMIP6_CV


# ==============================
#  main thread
# ==============================
def run():
    unittest.main()


class TestCase(base_CMIP6_CV.BaseCVsTest):
    def testCMIP6(self):
        # -------------------------------------------
        # Run CMOR using unicode strings
        # -------------------------------------------
        try:
            cmor.setup(inpath=u'Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
            cmor.dataset_json(u"Test/CMOR_input_example.json")
            cmor.load_table(u"CMIP6_CFsubhr.json")

            axes = [{u'table_entry': u'time1',
                    u'units': u'days since 2000-01-01 00:00:00',
                    },
                    {u'table_entry': u'site',
                    u'units': u'',
                    u'coord_vals': [0]},
                    {u'table_entry': u'hybrid_height',
                    u'units': u'm',
                    u'coord_vals': numpy.array(range(2), dtype=numpy.float64),
                    u'cell_bounds': [[x, x + 1] for x in range(2)],
                    },
                    ]

            values = numpy.array([0.5, 0.5], numpy.float32)

            axis_ids = list()
            for axis in axes:
                axis_id = cmor.axis(**axis)
                axis_ids.append(axis_id)

            igrid = cmor.grid([axis_ids[1]], [0.], [0.])
            cmor.zfactor(axis_ids[2], u'b', axis_ids=[axis_ids[2]],
                        zfactor_values=numpy.array(range(2), dtype=numpy.float64), 
                        zfactor_bounds=[[x, x + 1] for x in range(2)])

            cmor.zfactor(axis_ids[2], u'orog', u'm', axis_ids=[igrid],
                        zfactor_values=[0.] )

            ids_for_var = [axis_ids[0], igrid, axis_ids[2]]
            varid = cmor.variable(u'tnhus',
                                u's-1',
                                ids_for_var,
                                history=u'variable history',
                                missing_value=-99,
                                )

            for time in [x * 1800. / 86400 for x in range(48)]:
                cmor.write(varid, values, time_vals=[time])
            self.delete_files += [cmor.close(varid, True)]

            cmor.close()
        except BaseException:
            raise


if __name__ == '__main__':
    run()



import cmor

error_flag = cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)

error_flag = cmor.dataset_json("Test/CMOR_input_example.json")

cmor.load_table("CMIP6_Omon.json")
itim = cmor.axis(
    table_entry='time',
    units='months since 2010-1-1',
    coord_vals=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
    cell_bounds=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])

ivar = cmor.variable('thetaoga', units='deg_C', axis_ids=[itim, ])

data = [280., ] * 12  # 12 months worth of data

cmor.write(ivar, data)

cmor.close()



import json
import cmor
import unittest
import os

from netCDF4 import Dataset

DATASET_INFO = {
    "_AXIS_ENTRY_FILE": "Tables/CMIP6_coordinate.json",
    "_FORMULA_VAR_FILE": "Tables/CMIP6_formula_terms.json",
    "_controlled_vocabulary_file": "Tables/CMIP6_CV.json",
    "activity_id": "CMIP",
    "branch_method": "standard",
    "branch_time_in_child": 30.0,
    "branch_time_in_parent": 10800.0,
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "experiment": "AMIP",
    "experiment_id": "amip",
    "forcing_index": "3",
    "further_info_url": "https://furtherinfo.es-doc.org/CMIP6.MOHC.HadGEM3-GC31-LL.amip.none.r1i1p1f3",
    "grid": "N96",
    "grid_label": "gn",
    "initialization_index": "1",
    "institution": "Met Office Hadley Centre, Fitzroy Road, Exeter, Devon, EX1 3PB, UK",
    "institution_id": "MOHC",
    "license": "CMIP6 model data produced by the Met Office Hadley Centre is licensed under a Creative Commons Attribution-ShareAlike 4.0 International License (https://creativecommons.org/licenses). Consult https://pcmdi.llnl.gov/CMIP6/TermsOfUse for terms of use governing CMIP6 output, including citation requirements and proper acknowledgment. Further information about this data, including some limitations, can be found via the further_info_url (recorded as a global attribute in this file) and at https://ukesm.ac.uk/cmip6. The data producers and data providers make no warranty, either express or implied, including, but not limited to, warranties of merchantability and fitness for a particular purpose. All liabilities arising from the supply of the information (including any liability arising in negligence) are excluded to the fullest extent permitted by law.",
    "mip_era": "CMIP6",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "parent_activity_id": "no parent",
    "physics_index": "1",
    "realization_index": "1",
    "source": "HadGEM3-GC31-LL (2016): \naerosol: UKCA-GLOMAP-mode\natmos: MetUM-HadGEM3-GA7.1 (N96; 192 x 144 longitude/latitude; 85 levels; top level 85 km)\natmosChem: none\nland: JULES-HadGEM3-GL7.1\nlandIce: none\nocean: NEMO-HadGEM3-GO6.0 (eORCA1 tripolar primarily 1 deg with meridional refinement down to 1/3 degree in the tropics; 360 x 330 longitude/latitude; 75 levels; top grid cell 0-1 m)\nocnBgchem: none\nseaIce: CICE-HadGEM3-GSI8 (eORCA1 tripolar primarily 1 deg; 360 x 330 longitude/latitude)",
    "source_id": "HadGEM3-GC31-LL",
    "source_type": "AGCM",
    "sub_experiment": "none",
    "sub_experiment_id": "none",
    "tracking_prefix": "hdl:21.14100",
    "variant_label": "r1i1p1f3"
}


class TestBrandedVariable(unittest.TestCase):
    def setUp(self):
        """
        Write out a simple file using CMOR
        """
        # Set up CMOR
        cmor.setup(inpath="TestTables", netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile="cmor.log", create_subdirectories=0)

        # Define dataset using DATASET_INFO
        with open("Test/input_branded_variable.json", "w") as input_file_handle:
            json.dump(DATASET_INFO, input_file_handle, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json("Test/input_branded_variable.json")
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")


    def test_variable_without_branding_suffix(self):
        mip_table = "CMIP6_Omon_branded_variable.json"
        table_id = cmor.load_table(mip_table)

        itim = cmor.axis(
            table_entry='time',
            units='months since 2010-1-1',
            coord_vals=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            cell_bounds=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])
        ivar = cmor.variable('thetaoga', units='deg_C', axis_ids=[itim, ])

        data = [280., ] * 12
        cmor.write(ivar, data)
        filename = cmor.close(ivar, file_name=True)

        ds = Dataset(filename)
        attrs = ds.ncattrs()
        self.assertTrue('branding_suffix' not in attrs)
        self.assertTrue('temporal_label' not in attrs)
        self.assertTrue('vertical_label' not in attrs)
        self.assertTrue('horizontal_label' not in attrs)
        self.assertTrue('area_label' not in attrs)
        ds.close()
        os.remove(filename)


    def test_variable_with_branding_suffix(self):
        mip_table = "CMIP6_Omon_branded_variable.json"
        table_id = cmor.load_table(mip_table)

        itim = cmor.axis(
            table_entry='time',
            units='months since 2010-1-1',
            coord_vals=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            cell_bounds=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])
        ivar = cmor.variable('thetaoga_w1-x2-y3-z4', units='deg_C', axis_ids=[itim, ])

        data = [280., ] * 12
        cmor.write(ivar, data)
        filename = cmor.close(ivar, file_name=True)

        ds = Dataset(filename)
        attrs = ds.ncattrs()
        self.assertTrue('branding_suffix' in attrs)
        self.assertEqual('w1-x2-y3-z4', ds.getncattr('branding_suffix'))
        self.assertTrue('temporal_label' in attrs)
        self.assertEqual('w1', ds.getncattr('temporal_label'))
        self.assertTrue('vertical_label' in attrs)
        self.assertEqual('x2', ds.getncattr('vertical_label'))
        self.assertTrue('horizontal_label' in attrs)
        self.assertEqual('y3', ds.getncattr('horizontal_label'))
        self.assertTrue('area_label' in attrs)
        self.assertEqual('z4', ds.getncattr('area_label'))
        ds.close()
        os.remove(filename)


if __name__ == '__main__':
    unittest.main()



import cmor
import numpy
import sys
import os
from time import localtime, strftime
today = strftime("%Y%m%d", localtime())

try:
    import cdms2
except BaseException:
    print("This test code needs cdms2 interface for i/0")
    sys.exit()

cdms2.setNetcdfShuffleFlag(0)
cdms2.setNetcdfDeflateFlag(0)
cdms2.setNetcdfDeflateLevelFlag(0)

if len(sys.argv) > 1:
    level = int(sys.argv[1])
else:
    level = int(os.environ.get("DEFLATE_LEVEL", 0))

if len(sys.argv) > 2:
    shuffle = int(sys.argv[2])
else:
    shuffle = int(os.environ.get("SHUFFLE", 0))

if level == 0:
    deflate = 0
else:
    deflate = 1

f = open("Test/speed_test_table_A")
s = f.read()
f.close()
s = s.replace("${DEFLATE_LEVEL}", str(level))
s = s.replace("${DEFLATE}", str(deflate))
s = s.replace("${SHUFFLE}", str(shuffle))
f = open("mytable", "w")
f.write(s)
f.close()

cmor.setup(
    inpath="Test",
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE,
    exit_control=cmor.CMOR_EXIT_ON_MAJOR)
cmor.dataset(
    outpath="Test",
    experiment_id="2xCO2 equilibrium experiment",
    institution="GICC (Generic International Climate Center, Geneva, Switzerland)",
    source="GICCM1 (2002): atmosphere:  GICAM3 (gicam_0_brnchT_itea_2, T63L32); ocean: MOM (mom3_ver_3.5.2, 2x3L15); sea ice: GISIM4; land: GILSM2.5",
    calendar="standard",
    realization=1,
    contact="Charles Doutriaux (doutriaux1@llnl.gov)",
    history="Test for speed and compression.",
    comment="NetCDF4 vs NetCDF3 testing",
    references="http://cdat.sf.net",
    leap_year=0,
    leap_month=0,
    month_lengths=None, model_id="pcmdi-09a", forcing="co2")

tables = []
tables.append(cmor.load_table("mytable"))
print('Tables ids:', tables)


# read in data, just one slice
var = 'tos'
# f=cdms2.open('/export/ipcc/20c3m/atm/3h/tas/miroc3_2_medres/run1/tas_A3.nc')
# f=cdms2.open('/export/ipcc/20c3m/atm/3h/pr/miroc3_2_medres/run1/pr_A3.nc')
f = cdms2.open(
    '/export/ipcc/20c3m/ocn/mo/tos/miroc3_2_medres/run1/tos_O1_1934_2000.nc')
s = f[var]

ntimes = s.shape[0]
ntimes = 100
Tim = s.getTime().clone()
myaxes = numpy.arange(10)
myvars = numpy.arange(10)
try:
    missing_value = s.missing_value[0]
except BaseException:
    try:
        missing_value = s.missing_value
    except BaseException:
        missing_value = None

print('Missing:', type(missing_value), missing_value)
#missing_value = -999.
myaxes[0] = cmor.axis(table_entry='latitude',
                      units='degrees_north',
                      coord_vals=s.getLatitude()[:])
myaxes[1] = cmor.axis(table_entry='longitude',
                      units='degrees_north',
                      coord_vals=s.getLongitude()[:])


myaxes[2] = cmor.axis(table_entry='time',
                      units=Tim.units,
                      coord_vals=Tim[:ntimes])

pass_axes = [myaxes[2], myaxes[0], myaxes[1]]

myvars[0] = cmor.variable(table_entry=var,
                          units=s.units,
                          axis_ids=pass_axes,
                          original_name=var,
                          history='no history',
                          comment='testing speed',
                          missing_value=missing_value,
                          tolerance=1.e-2
                          )


import time
import MV2
st = time.time()
totcmor = 0
totcdms = 0
maxcmor = 0
mincmor = 1000
maxcdms = 0
mincdms = 1000
c0 = st
f = cdms2.open("Test/crap.nc", "w")
step = 10
for i in range(0, ntimes, step):
    print('Time:', i)
    j1 = i
    j2 = i + step
    cmor.write(myvars[0], s[j1:j2].filled(), step, file_suffix="speed-comp-02")
    c = time.time()
    # print 'cmor write time:',c-c0
    totcmor += c - c0
    if maxcmor < c - c0:
        maxcmor = c - c0
    if mincmor > c - c0:
        mincmor = c - c0
    c0 = c
    s2 = s[j1:j2]
    sh = list(s2.shape)
    if len(sh) == 2:
        sh.insert(0, 1)
        s2 = MV2.reshape(s2, sh)
        s2.setAxis(-1, s.getLongitude())
        s2.setAxis(-2, s.getLatitude())
        t = cdms2.createAxis([Tim[i], ])
        t.id = 'time'
        t.units = Tim.units
        t.designateTime()
        s2.setAxis(0, t)
    f.write(s2, id=var)
    c = time.time()
    # print 'cdms time:',c-c0
    totcdms += c - c0
    if maxcdms < c - c0:
        maxcdms = c - c0
    if mincdms > c - c0:
        mincdms = c - c0
    c0 = c
f.close()
cmor.close()

import cdtime
import os
ltime = cdtime.reltime(Tim[ntimes - 1], Tim.units).tocomp()
ftime = cdtime.reltime(Tim[0], Tim.units).tocomp()
print(ftime, ltime)
print(var, var, ftime.year, ftime.month, ltime.year, ltime.month)
lcmor = os.stat(
    "Test/IPCC_Fourth_Assessment/output/INSTITUTE_ID/pcmdi-09a/mon/REALM/%s/r1/%s_A1_pcmdi-09a_r1_%i%.2i-%i%.2i_speed-comp-02.nc" %
    (var, var, ftime.year, ftime.month, ltime.year, ltime.month))[6]
print('level:', level, "shuffle:", shuffle)
print('total cmor:', totcmor, mincmor, totcmor / ntimes, maxcmor, lcmor)
lcdms = os.stat("Test/crap.nc")[6]
print('total cdms:', totcdms, mincdms, totcdms / ntimes, maxcdms, lcdms)
print('Size diff:', float(lcmor) / float(lcdms))
print('speed diff:', totcmor / totcdms)

if os.path.exists("summary.txt"):
    f = open("summary.txt")
    s = f.read()
    f.close()
    dic = eval(s)
else:
    dic = {}

dic[(level, shuffle)] = (float(lcmor) / float(lcdms), totcmor / totcdms)

for i in range(10):
    a = dic.get((i, 0), "N/A")
    b = dic.get((i, 1), "N/A")
    print('Level: ', i, "no suffle:", a, "shuffle", b)
f = open("summary.txt", "w")
f.write(repr(dic))
f.close()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import os
import sys
import tempfile
import base_CMIP6_CV


# ==============================
#  main thread
# ==============================


def run():
    unittest.main()


class TestCase(base_CMIP6_CV.BaseCVsTest):

    def testCMIP6(self):

        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        nlat = 10
        dlat = 180. / nlat
        nlon = 20
        dlon = 360. / nlon

        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")
        cmor.load_table("CMIP6_fx.json")

        lats = numpy.arange(90 - dlat / 2., -90, -dlat)
        blats = numpy.arange(90, -90 - dlat, -dlat)
        lons = numpy.arange(0 + dlon / 2., 360., dlon)
        blons = numpy.arange(0, 360. + dlon, dlon)

        data = lats[:, numpy.newaxis] * lons[numpy.newaxis, :]

        data = (data + 1e10) / 750. + 233.2

        ilat = cmor.axis(
            table_entry='latitude',
            coord_vals=lats,
            cell_bounds=blats,
            units='degrees_north')
        ilon = cmor.axis(
            table_entry='longitude',
            coord_vals=lons,
            cell_bounds=blons,
            units='degrees_east')

        # ------------------------------------------
        # load Omon table and create masso variable
        # ------------------------------------------
        ivar = cmor.variable(
            table_entry="areacella", axis_ids=[
                ilat, ilon], units='m2')

        cmor.write(ivar, data)
        self.delete_files += [cmor.close(ivar, True)]
        cmor.close()


if __name__ == '__main__':
    run()



import cmor
import numpy
import os
import sys
import common

varin3d = ["MC", ]

n3d = len(varin3d)

pth = os.getcwd()
common.init_cmor(pth, "CMOR_input_example.json")
ierr = cmor.load_table(os.path.join(pth, 'Tables', 'CMIP6_Amon.json'))
itim, ilat, ilon = common.read_cmor_time_lat_lon()

ilev_half = cmor.axis(table_entry='standard_hybrid_sigma_half',
                      units='1',
                      coord_vals=common.zlevs)
print("ILEVL half:", ilev_half)

cmor.zfactor(zaxis_id=ilev_half, zfactor_name='p0',
             units='hPa', zfactor_values=common.p0)
print("p0 1/2")
cmor.zfactor(zaxis_id=ilev_half, zfactor_name='b_half', axis_ids=[ilev_half, ],
             zfactor_values=common.b_coeff)
print("b 1/2")
cmor.zfactor(zaxis_id=ilev_half, zfactor_name='a_half', axis_ids=[ilev_half, ],
             zfactor_values=common.a_coeff)
print("a 1/2")
ps_var = cmor.zfactor(zaxis_id=ilev_half, zfactor_name='ps',
                      axis_ids=[ilon, ilat, itim], units='hPa')
print("ps 1/2")

var3d_ids = []
for m in varin3d:
    print("3d VAR:", m)
    var3d_ids.append(
        cmor.variable(table_entry=common.specs[m]["entry"],
                      units=common.specs[m]["units"],
                      axis_ids=[itim, ilev_half, ilat, ilon],
                      missing_value=1.e28,
                      positive=common.specs[m]["positive"],
                      original_name=m)
    )
print("Ok now writing", var3d_ids, common.ntimes)
for index in range(common.ntimes):
    tim_array, bnds_tim = common.read_time(index)
    for i, varname in enumerate(varin3d):
        data = common.read_3d_input_files(
            index, varname, (common.lev, common.lat, common.lon))
        print(data.shape, data)
        print(tim_array, bnds_tim)
        cmor.write(var_id=var3d_ids[i], data=data, ntimes_passed=1,
                   time_vals=tim_array, time_bnds=bnds_tim)
        print("Passed write")
        # PSURF
        data = common.read_2d_input_files(
            index, "PSURF", (common.lat, common.lon))
        cmor.write(var_id=ps_var, data=data, ntimes_passed=1,
                   time_vals=tim_array, time_bnds=bnds_tim, store_with=var3d_ids[i])
cmor.close()



import cmor
import unittest
import numpy as np
from netCDF4 import Dataset
from base_CMIP6_CV import BaseCVsTest
from test_python_common import gen_irreg_grid, read_2d_input_files, \
                               Time, bnds_time, varin2d, lat, lon


class TestCRS(BaseCVsTest):

    def make_crs_file(self, crs_wkt=None):
        cmor.setup(inpath='cmip6-cmor-tables/Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE)
        cmor.dataset_json("Test/CMOR_input_example.json")

        grid_table = cmor.load_table("Tables/CMIP6_grids.json")
        lmon_table = cmor.load_table("Tables/CMIP6_Lmon.json")

        cmor.set_table(grid_table)

        x, y, lon_coords, lat_coords, lon_vertices, lat_vertices \
            = gen_irreg_grid(lon, lat)

        axes = [
                {
                    'table_entry': 'y',
                    'units': 'm',
                    'coord_vals': y
                },
                {
                    'table_entry': 'x',
                    'units': 'm',
                    'coord_vals': x
                }
            ]

        axis_ids = [cmor.axis(**axis) for axis in axes]

        grid_id = cmor.grid(axis_ids=axis_ids,
                            latitude=lat_coords,
                            longitude=lon_coords,
                            latitude_vertices=lat_vertices,
                            longitude_vertices=lon_vertices)
        axis_ids.append(grid_id)

        mapnm = 'lambert_conformal_conic'
        params = ["standard_parallel1",
                  "longitude_of_central_meridian",
                  "latitude_of_projection_origin",
                  "false_easting", "false_northing",
                  "standard_parallel2"]
        punits = ["", "", "", "", "", ""]
        pvalues = [-20., 175., 13., 8., 0., 20.]
        
        if crs_wkt is not None:
            params.append("crs_wkt")
            punits.append("")
            pvalues.append(crs_wkt)

        cmor.set_crs(grid_id=grid_id,
                     mapping_name=mapnm,
                     parameter_names=params,
                     parameter_values=pvalues,
                     parameter_units=punits)

        cmor.set_table(lmon_table)

        axis_ids.append(cmor.axis(table_entry='time',
                            units='months since 1980'))
        pass_axes = [axis_ids[3], axis_ids[2]]

        ivar = cmor.variable(table_entry='baresoilFrac',
                             units='',
                             axis_ids=pass_axes,
                             history='no history',
                             comment='no future'
                             )

        ntimes = 2
        for i in range(ntimes):
            data2d = read_2d_input_files(i, varin2d[0], lat, lon) * 1.E-6
            cmor.write(ivar, data2d, 1, time_vals=Time[i],
                    time_bnds=bnds_time[2 * i:2 * i + 2])

        filename = cmor.close(ivar, file_name=True)

        return filename


    def test_crs_without_crs_wkt(self):
        ds = Dataset(self.make_crs_file())

        self.assertTrue('crs' in ds.variables)
        attrs = ds.variables['crs'].ncattrs()
        test_attrs = {
            'grid_mapping_name': 'lambert_conformal_conic',
            'standard_parallel': np.asarray([-20., 20.]),
            'longitude_of_central_meridian': 175.0,
            'latitude_of_projection_origin': 13.0,
            'false_easting': 8.0,
            'false_northing': 0.0
        }

        for attr, val in test_attrs.items():
            self.assertTrue(attr in attrs)
            attr_val = ds.variables['crs'].getncattr(attr)
            if attr == 'standard_parallel':
                self.assertTrue(np.array_equal(val, attr_val))
            else:
                self.assertEqual(val, attr_val)

        self.assertTrue('crs_wkt' not in attrs)
        
        ds.close()


    def test_crs_with_crs_wkt(self):
        crs_wkt = "Text for the CRS WKT"
        ds = Dataset(self.make_crs_file(crs_wkt))

        self.assertTrue('crs' in ds.variables)
        attrs = ds.variables['crs'].ncattrs()
        test_attrs = {
            'grid_mapping_name': 'lambert_conformal_conic',
            'standard_parallel': np.asarray([-20., 20.]),
            'longitude_of_central_meridian': 175.0,
            'latitude_of_projection_origin': 13.0,
            'false_easting': 8.0,
            'false_northing': 0.0,
            'crs_wkt': crs_wkt
        }

        for attr, val in test_attrs.items():
            self.assertTrue(attr in attrs)
            attr_val = ds.variables['crs'].getncattr(attr)
            if attr == 'standard_parallel':
                self.assertTrue(np.array_equal(val, attr_val))
            else:
                self.assertEqual(val, attr_val)
        
        ds.close()


class TestLatLonGridMapping(BaseCVsTest):

    def test_latitude_longitude_grid_mapping(self):
        crs_wkt = \
            """
            GEOGCS[
                "WGS 84",
                DATUM[
                    "WGS_1984",
                    SPHEROID[
                        "WGS 84", 6378137, 298.257223563 ,
                        AUTHORITY["EPSG","7030"]
                    ],
                    AUTHORITY["EPSG", "6326"]
                ],
                PRIMEM["Greenwich", 0, AUTHORITY["EPSG", "8901"]],
                UNIT["degree", 0.0174532925199433, AUTHORITY["EPSG", "9122"]],
                AUTHORITY["EPSG", "4326"]
            ]
            """
        crs_params = {
            'grid_mapping_name': 'latitude_longitude',
            'longitude_of_prime_meridian': np.float64(0.0),
            'semi_major_axis': np.float64(6378137.0),
            'long_name': 'WGS 84',
            'inverse_flattening': np.float64(298.257223563),
            'GeoTransform': '-179.5 0.1 0 74.5 0.1',
            'crs_wkt': crs_wkt
            }
        
        grid_params = {
            'longitude_of_prime_meridian': [np.float64(0.0), "degrees_east"],
            'semi_major_axis': {"value": np.float64(6378137.0), "units": "meters"},
            'long_name': 'WGS 84',
            'inverse_flattening': (np.float64(298.257223563), ""),
            'GeoTransform': '-179.5 0.1 0 74.5 0.1',
            'crs_wkt': crs_wkt
            }

        cmor.setup(inpath='cmip6-cmor-tables/Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE)
        cmor.dataset_json("Test/CMOR_input_example.json")

        grid_table = cmor.load_table("Tables/CMIP6_grids.json")
        lmon_table = cmor.load_table("Tables/CMIP6_Lmon.json")

        cmor.set_table(grid_table)

        nlat = 10
        nlon = 20
        lon_coords = np.arange(-120., -120. + 1.*nlon, 1.) + 0.5
        lon_coords = lon_coords % 360.
        lat_coords = np.arange(60., 60 + 1.*nlat, 1.) + 0.5
        lon_bounds = np.zeros((lon_coords.shape[0], 2))
        lon_bounds[:, 0] = lon_coords - 0.5
        lon_bounds[:, 1] = lon_coords + 0.5
        lat_bounds = np.zeros((lat_coords.shape[0], 2))
        lat_bounds[:, 0] = lat_coords - 0.5
        lat_bounds[:, 1] = lat_coords + 0.5

        lat_grid, lon_grid = np.broadcast_arrays(
            np.expand_dims(lat_coords, 0),
            np.expand_dims(lon_coords, 1)
            )

        axes = [
                {
                    "table_entry": "latitude",
                    "units": "degrees_north",
                    "coord_vals": lat_coords,
                    "cell_bounds": lat_bounds
                },
                {
                    "table_entry": "longitude",
                    "units": "degrees_east",
                    "coord_vals": lon_coords,
                    "cell_bounds": lon_bounds
                }
            ]

        axis_ids = [cmor.axis(**axis) for axis in axes]

        grid_id = cmor.grid(axis_ids=axis_ids,
                            latitude=lat_grid,
                            longitude=lon_grid)
        axis_ids.append(grid_id)
        cmor.set_crs(grid_id=grid_id,
                     mapping_name=crs_params['grid_mapping_name'],
                     parameter_names=grid_params)

        cmor.set_table(lmon_table)

        axis_ids.append(cmor.axis(table_entry='time',
                            units='months since 1980'))
        pass_axes = [axis_ids[3], axis_ids[2]]

        ivar = cmor.variable(table_entry='baresoilFrac',
                             units='',
                             axis_ids=pass_axes,
                             history='no history',
                             comment='no future'
                             )

        ntimes = 2
        for i in range(ntimes):
            data2d = read_2d_input_files(i, varin2d[0], nlat, nlon) * 1.E-6
            cmor.write(ivar, data2d, 1, time_vals=Time[i],
                    time_bnds=bnds_time[2 * i:2 * i + 2])

        filename = cmor.close(ivar, file_name=True)
        ds = Dataset(filename)

        self.assertTrue('crs' in ds.variables)
        attrs = ds.variables['crs'].ncattrs()

        for attr, val in crs_params.items():
            self.assertTrue(attr in attrs)
            attr_val = ds.variables['crs'].getncattr(attr)
            self.assertEqual(val, attr_val)
        
        ds.close()


class TestStandardParallel(BaseCVsTest):

    def test_grid_mapping_without_standard_parallel(self):
        cmor.setup(inpath='cmip6-cmor-tables/Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        grid_table = cmor.load_table("Tables/CMIP6_grids.json")

        cmor.set_table(grid_table)

        x, y, lon_coords, lat_coords, lon_vertices, lat_vertices \
            = gen_irreg_grid(lon, lat)

        axes = [
                {
                    'table_entry': 'y',
                    'units': 'm',
                    'coord_vals': y
                },
                {
                    'table_entry': 'x',
                    'units': 'm',
                    'coord_vals': x
                }
            ]

        axis_ids = [cmor.axis(**axis) for axis in axes]

        grid_id = cmor.grid(axis_ids=axis_ids,
                            latitude=lat_coords,
                            longitude=lon_coords,
                            latitude_vertices=lat_vertices,
                            longitude_vertices=lon_vertices)
        axis_ids.append(grid_id)

        mapnm = 'lambert_conformal_conic'
        params = ["longitude_of_central_meridian",
                  "latitude_of_projection_origin",
                  "false_easting",
                  "false_northing"]
        punits = ["", "", "", ""]
        pvalues = [175., 13., 8., 0.]

        cmor.set_crs(grid_id=grid_id,
                     mapping_name=mapnm,
                     parameter_names=params,
                     parameter_values=pvalues,
                     parameter_units=punits)

        self.assertCV(
            ("you should consider setting standard_parallel for one value "
             "or standard_parallel1 and standard_parallel2 for two values"),
            ("Warning: Grid mapping attribute standard_parallel "
             "has not been set, ")
        )


class TestPolarStereographicAttributeOptions(BaseCVsTest):

    def setUp(self, *args, **kwargs):

        # Create x and y coordinates and bounds.
        coord_vals = np.arange(-533750.0, 533750.0 + 2500.0,
                               2500.0, np.float32)
        coord_bnds = np.zeros((coord_vals.shape[0], 2), np.float32)

        coord_bnds[:, 0] = coord_vals - 1250.0
        coord_bnds[:, 1] = coord_vals + 1250.0

        # Create longitude and latitude fields for a
        # polar stereographic projection.
        xgrid, ygrid = np.broadcast_arrays(
            np.expand_dims(
                coord_vals, 0), np.expand_dims(
                coord_vals, 1))

        rhogrid = np.sqrt(xgrid**2 + ygrid**2)
        cgrid = 2.0 * np.arctan((0.5 / 6378137.0) * rhogrid)
        self.lat_grid = (180.0 / 3.141592654) * np.arcsin(np.cos(cgrid))
        self.lon_grid = (180.0 / 3.141592654) * np.arctan2(xgrid, -ygrid)

        self.axes = [
                {
                    'table_entry': 'y',
                    'units': 'm',
                    'coord_vals': coord_vals,
                    'cell_bounds': coord_bnds
                },
                {
                    'table_entry': 'x',
                    'units': 'm',
                    'coord_vals': coord_vals,
                    'cell_bounds': coord_bnds
                }
            ]

        return super().setUp(*args, **kwargs)

    def test_polar_stereographic_without_attribute_options(self):

        cmor.setup(inpath='cmip6-cmor-tables/Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        grid_table = cmor.load_table("Tables/CMIP6_grids.json")

        cmor.set_table(grid_table)

        axis_ids = [cmor.axis(**axis) for axis in self.axes]

        grid_id = cmor.grid(axis_ids=axis_ids,
                            latitude=self.lat_grid,
                            longitude=self.lon_grid)
        axis_ids.append(grid_id)

        mapnm = 'polar_stereographic'
        param_dict = {
            'latitude_of_projection_origin': [90.0, 'degrees_north'],
            'longitude_of_projection_origin': [135.0, 'degrees_east'],
            'false_northing': [0.0, 'meters'],
            'false_easting': [0.0, 'meters']
        }

        cmor.set_crs(grid_id=grid_id,
                     mapping_name=mapnm,
                     parameter_names=param_dict)

        self.assertCV(
            ("You should consider setting one of the attributes"),
            ("Warning: Grid mapping attributes standard_parallel or "
             "scale_factor_at_projection_origin were not set for "
             f"grid mapping {mapnm}. ")
        )

    def test_polar_stereographic_with_both_attribute_options(self):

        cmor.setup(inpath='cmip6-cmor-tables/Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        grid_table = cmor.load_table("Tables/CMIP6_grids.json")

        cmor.set_table(grid_table)

        axis_ids = [cmor.axis(**axis) for axis in self.axes]

        grid_id = cmor.grid(axis_ids=axis_ids,
                            latitude=self.lat_grid,
                            longitude=self.lon_grid)
        axis_ids.append(grid_id)

        mapnm = 'polar_stereographic'
        param_dict = {
            'latitude_of_projection_origin': [90.0, 'degrees_north'],
            'longitude_of_projection_origin': [135.0, 'degrees_east'],
            'standard_parallel': [70.0, 'degrees_north'],
            'scale_factor_at_projection_origin': [1.0, 'm'],
            'false_northing': [0.0, 'meters'],
            'false_easting': [0.0, 'meters']
        }

        cmor.set_crs(grid_id=grid_id,
                     mapping_name=mapnm,
                     parameter_names=param_dict)

        self.assertCV(
            ("You should consider setting only one of the attributes"),
            ("Warning: Both grid mapping attributes standard_parallel and "
             "scale_factor_at_projection_origin were set for "
             f"grid mapping {mapnm}. ")
        )

    def test_polar_stereographic_attribute_with_wrong_units(self):

        cmor.setup(inpath='cmip6-cmor-tables/Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        grid_table = cmor.load_table("Tables/CMIP6_grids.json")

        cmor.set_table(grid_table)

        axis_ids = [cmor.axis(**axis) for axis in self.axes]

        grid_id = cmor.grid(axis_ids=axis_ids,
                            latitude=self.lat_grid,
                            longitude=self.lon_grid)
        axis_ids.append(grid_id)

        mapnm = 'polar_stereographic'
        param_dict = {
            'latitude_of_projection_origin': [90.0, 'degrees_north'],
            'longitude_of_projection_origin': [135.0, 'degrees_east'],
            'scale_factor_at_projection_origin': [1.0, 'xyz'],
            'false_northing': [0.0, 'meters'],
            'false_easting': [0.0, 'meters']
        }

        with self.assertRaises(cmor.CMORError):
            _ = cmor.set_crs(grid_id=grid_id,
                             mapping_name=mapnm,
                             parameter_names=param_dict)

        self.assertCV(
            "(xyz)",
            "Error: Udunits: analyzing units from cmor "
        )
        self.assertCV(
            "xyz and m",
            "Error: Udunits: Error preparing units "
        )


if __name__ == '__main__':
    unittest.main()



import cmor
import numpy
import unittest
import os
import shutil
import tempfile
from collections import namedtuple
from netCDF4 import Dataset
from base_CMIP6_CV import BaseCVsTest


ZSTDSettings = namedtuple('ZSTDSettings', ['level'])
DeflateSettings = namedtuple('DeflateSettings', ['deflate', 'level', 'shuffle'])
QuantizeSettings = namedtuple('QuantizeSettings', ['mode', 'nsd'])


def run():
    unittest.main()


class TestCase(BaseCVsTest):

    def gen_file(self, seed, out_dir, ntimes, 
                 zstd_settings: ZSTDSettings = None,
                 deflate_settings: DeflateSettings = None,
                 quantize_settings: QuantizeSettings = None):

        numpy.random.seed(seed)

        cmor.setup(inpath='Test',
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        cmor.dataset_json("Test/CMOR_input_example.json")

        # creates 1 degree grid
        nlat = 18
        nlon = 36
        alats = numpy.arange(180) - 89.5
        bnds_lat = numpy.arange(181) - 90
        alons = numpy.arange(360) + .5
        bnds_lon = numpy.arange(361)
        cmor.load_table("Tables/CMIP6_Amon.json")
        ilat = cmor.axis(
            table_entry='latitude',
            units='degrees_north',
            length=nlat,
            coord_vals=alats,
            cell_bounds=bnds_lat)

        ilon = cmor.axis(
            table_entry='longitude',
            length=nlon,
            units='degrees_east',
            coord_vals=alons,
            cell_bounds=bnds_lon)

        plevs = numpy.array([100000., 92500, 85000, 70000, 60000, 50000, 40000,
                            30000, 25000, 20000, 15000, 10000, 7000, 5000, 3000,
                            2000, 1000, 999, 998, 997, 996, 995, 994, 500, 100])

        itim = cmor.axis(
            table_entry='time',
            units='months since 2030-1-1',
            length=ntimes,
            interval='1 month')

        ilev = cmor.axis(
            table_entry='plev19',
            units='Pa',
            coord_vals=plevs,
            cell_bounds=None)

        var3d_ids = cmor.variable(
            table_entry='ta',
            units='K',
            axis_ids=numpy.array((ilev, ilon, ilat, itim)),
            missing_value=numpy.array([1.0e28, ], dtype=numpy.float32)[0],
            original_name='cloud')

        if quantize_settings is not None:
            cmor.set_quantize(var3d_ids, quantize_settings.mode, quantize_settings.nsd)

        if deflate_settings is not None:
            use_deflate = 1 if deflate_settings.deflate > 0 else 0
            use_shuffle = 1 if deflate_settings.shuffle else 0
            cmor.set_deflate(var3d_ids, use_shuffle, use_deflate,
                            deflate_level=deflate_settings.level)
        
        if zstd_settings is not None:
            cmor.set_zstandard(var3d_ids, zstd_settings.level)

        for it in range(ntimes):

            time = numpy.array((it))
            bnds_time = numpy.array((it, it + 1))
            data3d = numpy.random.random((len(plevs), nlon, nlat)) * 30. + 265.
            data3d = data3d.astype('f')
            cmor.write(
                var_id=var3d_ids,
                data=data3d,
                ntimes_passed=1,
                time_vals=time,
                time_bnds=bnds_time)

        nc_path = cmor.close(var3d_ids, file_name=True)

        dst_file = 'test_data'
        if deflate_settings is not None:
            if deflate_settings.deflate:
                dst_file += f'_deflate_level_{str(deflate_settings.level)}'
        else:
            dst_file += f'_deflate_level_1'

        if zstd_settings is not None:
            dst_file += f'_zstd_level_{str(zstd_settings.level)}'
    
        if deflate_settings is not None and deflate_settings.shuffle:
            dst_file += '_shuffle'
        if quantize_settings is not None:
            dst_file += f'_qmode_{str(quantize_settings.mode)}_nsd_{str(quantize_settings.nsd)}'
        dst_file += '.nc'

        dst_path = os.path.join(out_dir, dst_file)

        shutil.copy(nc_path, dst_path)

        return dst_path


    def test_zstandard_compression(self):

        seed = 123
        ntimes = 100

        with tempfile.TemporaryDirectory() as tmp_dir:
            default_deflate = self.gen_file(seed, tmp_dir, ntimes)
            no_compression = self.gen_file(seed, tmp_dir, ntimes,
                                           deflate_settings=DeflateSettings(False, 0, False))
            zstd_shuffle = self.gen_file(seed, tmp_dir, ntimes, 
                                         zstd_settings=ZSTDSettings(3),
                                         deflate_settings=DeflateSettings(False, 0, True))

            no_comp_size = os.path.getsize(no_compression)
            zstd_shuffle_size = os.path.getsize(zstd_shuffle)
            print(f'File size without compression: {no_comp_size} bytes')
            print(f'File size with zstandard compression and shuffle: {zstd_shuffle_size} bytes')
            self.assertTrue(zstd_shuffle_size < no_comp_size)

            default_deflate_nc = Dataset(default_deflate, "r", format="NETCDF4")
            no_comp_nc = Dataset(no_compression, "r", format="NETCDF4")
            zstd_shuffle_nc = Dataset(zstd_shuffle, "r", format="NETCDF4")

            default_deflate_ta = default_deflate_nc.variables['ta'][:]
            no_comp_ta = no_comp_nc.variables['ta'][:]
            zstd_shuffle_ta = zstd_shuffle_nc.variables['ta'][:]
            self.assertIsNone(numpy.testing.assert_array_equal(default_deflate_ta, zstd_shuffle_ta))
            self.assertIsNone(numpy.testing.assert_array_equal(no_comp_ta, zstd_shuffle_ta))

            default_deflate_filters = default_deflate_nc.variables['ta'].filters()
            self.assertTrue(default_deflate_filters['zlib'])
            self.assertFalse(default_deflate_filters['zstd'])
            self.assertFalse(default_deflate_filters['shuffle'])
            self.assertEqual(default_deflate_filters['complevel'], 1)

            no_comp_filters = no_comp_nc.variables['ta'].filters()
            self.assertFalse(no_comp_filters['zlib'])
            self.assertFalse(no_comp_filters['zstd'])
            self.assertFalse(no_comp_filters['shuffle'])
            self.assertEqual(no_comp_filters['complevel'], 0)

            zstd_shuffle_filters = zstd_shuffle_nc.variables['ta'].filters()
            self.assertFalse(zstd_shuffle_filters['zlib'])
            self.assertTrue(zstd_shuffle_filters['zstd'])
            self.assertTrue(zstd_shuffle_filters['shuffle'])
            self.assertEqual(zstd_shuffle_filters['complevel'], 3)


    def test_no_zstandard_compression_if_deflate_still_enabled(self):

        seed = 123
        ntimes = 100

        with tempfile.TemporaryDirectory() as tmp_dir:
            filename = self.gen_file(seed, tmp_dir, ntimes,
                                     zstd_settings=ZSTDSettings(3))

            nc = Dataset(filename, "r", format="NETCDF4")

            filters = nc.variables['ta'].filters()
            self.assertTrue(filters['zlib'])
            self.assertFalse(filters['zstd'])
            self.assertFalse(filters['shuffle'])
            self.assertEqual(filters['complevel'], 1)


    def test_no_zstandard_compression_if_level_out_of_range(self):

        seed = 123
        ntimes = 100

        with tempfile.TemporaryDirectory() as tmp_dir:
            with self.assertRaises(cmor.CMORError):
                _ = self.gen_file(seed, tmp_dir, ntimes,
                                  zstd_settings=ZSTDSettings(100),
                                  deflate_settings=DeflateSettings(False, 0, True))
            
            self.assertCV("Your zstandard level of 100 is outside of the range [-131072, 22].")


    def test_quantize(self):

        seed = 123
        ntimes = 100

        with tempfile.TemporaryDirectory() as tmp_dir:
            default = self.gen_file(seed, tmp_dir, ntimes,
                                    quantize_settings=QuantizeSettings(0, 0))
            quantized = self.gen_file(seed, tmp_dir, ntimes,
                                      quantize_settings=QuantizeSettings(1, 4))

            default_size = os.path.getsize(default)
            quantized_size = os.path.getsize(quantized)
            print(f'File size without quantization: {default_size} bytes')
            print(f'File size with quantization: {quantized_size} bytes')
            self.assertTrue(quantized_size < default_size)

            default_nc = Dataset(default, "r", format="NETCDF4")
            quantized_nc = Dataset(quantized, "r", format="NETCDF4")

            default_ta = default_nc.variables['ta']
            quantized_ta = quantized_nc.variables['ta']
            self.assertIsNone(
                numpy.testing.assert_allclose(default_ta[:],
                                              quantized_ta[:],
                                              rtol=1e-4)
            )

            default_attributes = default_ta.ncattrs()

            self.assertTrue('quantization_info' not in default_nc.variables)
            self.assertTrue('quantization' not in default_attributes)
            self.assertTrue('quantization_nsd' not in default_attributes)
            self.assertTrue('quantization_nsb' not in default_attributes)

            quantized_attributes = quantized_ta.ncattrs()

            self.assertTrue('quantization_info' in quantized_nc.variables)
            self.assertTrue('quantization' in quantized_attributes)
            self.assertTrue('quantization_nsd' in quantized_attributes)
            self.assertTrue('quantization_nsb' not in quantized_attributes)

            quantized_attr = quantized_ta.getncattr('quantization')
            quantized_nsd = quantized_ta.getncattr('quantization_nsd')

            self.assertEqual(quantized_attr, 'quantization_info')
            self.assertEqual(quantized_nsd, 4)

            quantized_info = quantized_nc.variables['quantization_info']
            quantized_alg = quantized_info.getncattr('algorithm')
            quantized_impl = quantized_info.getncattr('implementation')

            self.assertEqual(quantized_alg, 'bitgroom')
            self.assertTrue('libnetcdf version ' in quantized_impl)


    def test_quantization_modes(self):

        seed = 123
        ntimes = 100

        with tempfile.TemporaryDirectory() as tmp_dir:
            granular = self.gen_file(seed, tmp_dir, ntimes,
                                     quantize_settings=QuantizeSettings(2, 3))
            bitround = self.gen_file(seed, tmp_dir, ntimes,
                                     quantize_settings=QuantizeSettings(3, 6))

            granular_nc = Dataset(granular, "r", format="NETCDF4")
            bitround_nc = Dataset(bitround, "r", format="NETCDF4")

            granular_ta = granular_nc.variables['ta']
            bitround_ta = bitround_nc.variables['ta']

            granular_attributes = granular_ta.ncattrs()

            self.assertTrue('quantization_info' in granular_nc.variables)
            self.assertTrue('quantization' in granular_attributes)
            self.assertTrue('quantization_nsd' in granular_attributes)
            self.assertTrue('quantization_nsb' not in granular_attributes)

            granular_attr = granular_ta.getncattr('quantization')
            granular_nsd = granular_ta.getncattr('quantization_nsd')

            self.assertEqual(granular_attr, 'quantization_info')
            self.assertEqual(granular_nsd, 3)

            granular_info = granular_nc.variables['quantization_info']
            granular_alg = granular_info.getncattr('algorithm')
            granular_impl = granular_info.getncattr('implementation')

            self.assertEqual(granular_alg, 'granular_bitround')
            self.assertTrue('libnetcdf version ' in granular_impl)

            bitround_attributes = bitround_ta.ncattrs()

            self.assertTrue('quantization_info' in bitround_nc.variables)
            self.assertTrue('quantization' in bitround_attributes)
            self.assertTrue('quantization_nsd' not in bitround_attributes)
            self.assertTrue('quantization_nsb' in bitround_attributes)

            bitround_attr = bitround_ta.getncattr('quantization')
            bitround_nsd = bitround_ta.getncattr('quantization_nsb')

            self.assertEqual(bitround_attr, 'quantization_info')
            self.assertEqual(bitround_nsd, 6)

            bitround_info = bitround_nc.variables['quantization_info']
            bitround_alg = bitround_info.getncattr('algorithm')
            bitround_impl = bitround_info.getncattr('implementation')

            self.assertEqual(bitround_alg, 'bitround')
            self.assertTrue('libnetcdf version ' in bitround_impl)


if __name__ == '__main__':
    run()



import cmor
import numpy
import unittest
from base_CMIP6_CV import BaseCVsTest


class TestUnsupportedCalendar(BaseCVsTest):

    def test_unsupported_calendar(self):

        cmor.setup(inpath='Tables',
                    netcdf_file_action=cmor.CMOR_REPLACE,
                    logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")
        cmor.set_cur_dataset_attribute("calendar", "366_day")

        cmor.load_table("CMIP6_Omon.json")

        with self.assertRaises(cmor.CMORError):
            _ = cmor.axis(table_entry="time", units='months since 2010',
                          coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                          cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))

        self.assertCV(("A 366_day calendar is an invalid selection for "
                       "Model Intercomparison Project (MIP) data."))


if __name__ == '__main__':
    unittest.main()



import unittest
import os
import tempfile
import glob

debug = False


class BaseCVsTest(unittest.TestCase):
    def remove_file_and_directories(self, filename):
        os.remove(filename)
        filename = os.path.dirname(filename)
        while glob.glob(os.path.join(filename, "*")) == []:
            os.rmdir(filename)
            filename = os.path.dirname(filename)

    def setUp(self, *args, **kwargs):
        # --------------
        # Create tmpfile
        # --------------
        self.tmpfile = tempfile.mkstemp()[1]
        if debug:
            print("TEMP:", self.tmpfile)
        self.delete_files = []

    def tearDown(self):
        if debug:
            print("would be unlinking:", self.tmpfile)
            return
        os.unlink(self.tmpfile)
        for filename in self.delete_files:
            self.remove_file_and_directories(filename)
            self.delete_files.remove(filename)

    def assertCV(self, text_to_find, line_trigger='Error:', number_of_lines_to_scan=1):
        line_to_scan = ""
        if debug:
            print("LINE TRIGGER:", line_trigger)
        with open(self.tmpfile) as f:
            lines = f.readlines()
            for i, line in enumerate(lines):
                scan = "".join(lines[i:i+number_of_lines_to_scan]).strip()
                if debug: print("LINE:", i, __file__, scan)
                if line_trigger in line:
                    line_to_scan = scan
                    break
        if debug:
            print("SCANNED:", line_to_scan)
        self.assertIn(text_to_find, line_to_scan)


import cmor
import numpy
import os
ntimes = 2
lon = 300
lat = 100
lev = 5


def read_time(it):
    time = [0]
    time_bnds = [0, 0]
    time[0] = (it - 0.5) * 30.
    time_bnds[0] = (it - 1) * 30.
    time_bnds[1] = it * 30.

    time[0] = it
    time_bnds[0] = it
    time_bnds[1] = it + 1
    return time[0], numpy.array(time_bnds)


def gen_irreg_grid(lon, lat):
    lon0 = 5.
    lat0 = -17.5
    delta_lon = .1
    delta_lat = .1
    y = numpy.arange(lat)
    x = numpy.arange(lon)
    lon_coords = numpy.zeros((lat, lon))
    lat_coords = numpy.zeros((lat, lon))
    lon_vertices = numpy.zeros((lat, lon, 4))
    lat_vertices = numpy.zeros((lat, lon, 4))

    for j in range(lat):  # really porr coding i know
        for i in range(lon):  # getting worse i know
            lon_coords[j, i] = lon0 + delta_lon * (j + 1 + i)
            lat_coords[j, i] = lat0 + delta_lat * (j + 1 - i)
            lon_vertices[j, i, 0] = lon_coords[j, i] - delta_lon
            lon_vertices[j, i, 1] = lon_coords[j, i]
            lon_vertices[j, i, 2] = lon_coords[j, i] + delta_lon
            lon_vertices[j, i, 3] = lon_coords[j, i]
# !!$      /* vertices lat */
            lat_vertices[j, i, 0] = lat_coords[j, i]
            lat_vertices[j, i, 1] = lat_coords[j, i] - delta_lat
            lat_vertices[j, i, 2] = lat_coords[j, i]
            lat_vertices[j, i, 3] = lat_coords[j, i] + delta_lat
    print(lat_vertices.min(), '---------------------')
    return x, y, lon_coords, lat_coords, lon_vertices, lat_vertices


myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')


cmor.setup(
    inpath="Tables",
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE,
    exit_control=cmor.CMOR_EXIT_ON_MAJOR)
cmor.dataset_json("Test/CMOR_input_example.json")

tables = []
a = cmor.load_table("CMIP6_grids.json")
tables.append(a)
tables.append(cmor.load_table("CMIP6_Omon.json"))
print('Tables ids:', tables)

cmor.set_table(tables[0])

x, y, lon_coords, lat_coords, lon_vertices, lat_vertices = gen_irreg_grid(
    lon, lat)
print(lon_vertices.shape, lat_vertices.shape, x.shape, y.shape)

myaxes[1] = cmor.axis(table_entry='y',
                      units='m',
                      coord_vals=y)
myaxes[0] = cmor.axis(table_entry='x',
                      units='m',
                      coord_vals=x)

print('lons:', lon_vertices.shape, lon_coords.shape)
grid_id = cmor.grid(axis_ids=myaxes[:2],
                    latitude=lat_coords,
                    longitude=lon_coords,
                    latitude_vertices=lat_vertices,
                    longitude_vertices=lon_vertices)
print('got grid_id:', grid_id)
myaxes[2] = grid_id

## mapnm = 'lambert_conformal_conic'
# params = [ "standard_parallel1",
# "longitude_of_central_meridian","latitude_of_projection_origin",
# "false_easting","false_northing","standard_parallel2" ]
## punits = ["","","","","","" ]
## pvalues = [-20.,175.,13.,8.,0.,20. ]
# cmor.set_grid_mapping(grid_id=myaxes[2],
##                       mapping_name = mapnm,
##                       parameter_names = params,
##                       parameter_values = pvalues,
# parameter_units = punits)

cmor.set_table(tables[1])
myaxes[3] = cmor.axis(table_entry='time',
                      units='months since 1980')
# Now sets up the ocn sigma stuff
levs = -numpy.arange(lev) / float(lev + 1.)
blevs = -numpy.arange(lev + 1) / float(lev + 1.)
print('Defining zlevs')
myaxes[4] = cmor.axis(
    table_entry='ocean_sigma',
    coord_vals=levs,
    cell_bounds=blevs,
    units='1')

print('definnig zfactor depth', myaxes[2])
depth = numpy.random.random((lon, lat)) * 5000.
print('Depth:', depth.shape, depth.dtype)
idpth = cmor.zfactor(zaxis_id=myaxes[4],
                     units='m',
                     zfactor_name='depth',
                     axis_ids=numpy.array([myaxes[2],
                                           ]),
                     zfactor_values=depth)

print('defining zfactor eta')
ieta = cmor.zfactor(
    zaxis_id=myaxes[4],
    units='m',
    zfactor_name='eta',
    axis_ids=[
        myaxes[2],
        myaxes[3]])
print('ieta:', ieta)
pass_axes = [myaxes[4], myaxes[2], myaxes[3]]
print('defining variable')
myvars[0] = cmor.variable(table_entry='thetao',
                          units='K',
                          axis_ids=pass_axes,
                          positive='down'
                          )
Time = numpy.zeros(ntimes, dtype='d')
bnds_time = numpy.zeros(ntimes * 2, dtype='d')
Time[0], bnds_time[0:2] = read_time(0)
Time[1], bnds_time[2:4] = read_time(1)
for i in range(ntimes):
    data3d = numpy.random.random((lev, lon, lat, ntimes)) * 40. + 273.15
    eta = numpy.random.random((lon, lat, ntimes)) * 10000.
    # print 'writing time: ',i,data3d.shape,data3d
    # print Time[i],bnds_time[2*i:2*i+2]
    print('Writing time', i, 'for var', data3d.shape)
    cmor.write(myvars[0], data3d, 1, time_vals=Time[i],
               time_bnds=bnds_time[2 * i:2 * i + 2])
    print('Writing time', i, 'for eta')
    cmor.write(ieta,
               eta,
               1,
               time_vals=Time[i],
               time_bnds=bnds_time[2 * i:2 * i + 2],
               store_with=myvars[0])
cmor.close()



import csv
import re
import pdb
# ==============================================================
#                     replaceString()
# ==============================================================


def replaceString(var_entry, var, field):
    """
       Replace a <field> with a string (deleting "<" and ">")
    """
    if(var == ''):
        var_entry = re.sub(r"" + field + ":.*\n", "", var_entry)
    var_entry = var_entry.replace("<" + field + ">", var)
    return var_entry
# ==============================================================
#                     deleteLIne()
# ==============================================================


def deleteLine(var_entry, field):
    """
       delete line with <field>  in it
    """
    var_entry = re.sub(r"\n.*" + field + ".*\n", "\n", var_entry)
    return var_entry

# ==============================================================
#                     deleteComa()
# ==============================================================


def deleteComa(var_entry):
    """
       delete last ',' char before '}'
    """
    var_entry = re.sub(r',(\s*)}', r'\n\1}', var_entry)
    return var_entry


expt_template = """
        "<experiment_id>": {
                               "experiment":                "<title>",
                               "sub_experiment_id":         "<sub_experiment_id>",
                               "activity_id":               "<mip>",
                               "mip_era":                   "CMIP6",
                               "source_type":               "<required_source_type>",
                               "additional_source_type":    "<add_source_type>",
                               "parent_experiment_id" :     "<parent_experiment_id>",
                               "parent_sub_experiment_id":  "<parent_sub_experiment_id>",
                               "parent_activity_id ":       "<parent_activity_id>",
                               "parent_mip_era ":           "CMIP6"
                          },
"""
print("{")
print("    \"experiment_ids\": { ")
with open('../Tables/CMIP6_expt_list_062116.csv', 'rU') as csvfile:
    spamreader = csv.reader(csvfile, dialect=csv.excel)
    for row in spamreader:
        if row[1] == 'original label':
            break
    expt = ""
    for row in spamreader:
        # if (row[5] == "") & (row[22] != ""):
        # print i
        if (row[9] != ""):
            expt = expt + expt_template
            expt = replaceString(expt, row[9], "experiment_id")
            expt = replaceString(expt, row[17].replace('"', '\''), "title")
            expt = replaceString(
                expt, row[18].replace(
                    '"', '\''), "sub_experiment_id")
            expt = replaceString(expt, row[2], "mip")
            expt = replaceString(expt, row[14], "required_source_type")
            expt = replaceString(expt, row[15], "add_source_type")
            expt = replaceString(expt, row[20], "parent_sub_experiment_id")
            expt = replaceString(expt, row[21], "parent_activity_id")

            if (row[19] != ""):
                expt = replaceString(expt, row[19], "parent_experiment_id")
            else:
                expt = deleteLine(expt, "parent_experiment_id")
                expt = deleteLine(expt, "parent_sub_experiment_id")
                expt = deleteLine(expt, "parent_activity_id")
                expt = deleteLine(expt, "parent_mip_era")
                expt = deleteComa(expt)


#nexpt = expt + "\"Dummy\":{}\n     }"
expt = expt + "\n    }\n}"
expt = deleteComa(expt)
print(expt)
